@@ -3,7 +3,7 @@
 #include <math.h>
 #include <stdarg.h>
 #include <string.h>
-
+#include <ctype.h>
 
 #include "io_lib_header.h"
 #include "util_lib_header.h"
@@ -315,7 +315,7 @@ int aln_compare ( int argc, char *argv[])
 	else
 		{
 		fprintf ( stdout, "\nOPTION %s UNKNOWN[FATAL]\n", argv[a]);
-		exit (EXIT_FAILURE);
+		myexit (EXIT_FAILURE);
 		}
 	}
  
@@ -331,22 +331,22 @@ if ( aln_compare==1)pep_compare=0;
         if (  alignment1_file[0] && !check_file_exists ( alignment1_file))
 	 {
 	   fprintf (stderr, "\nERROR: %s DOES NOT EXIST[FATAL:%s]\n",  alignment1_file, PROGRAM);
-	   exit(EXIT_FAILURE);
+	   myexit(EXIT_FAILURE);
 	 } 
 	if (  alignment2_file[0] && !check_file_exists ( alignment2_file))
 	 {
 	   fprintf (stderr, "\nERROR: %s DOES NOT EXIST[FATAL:%s]\n",  alignment2_file, PROGRAM);
-	   exit(EXIT_FAILURE);
+	   myexit(EXIT_FAILURE);
 	 }
 	if (  pep1_file[0] && !check_file_exists ( pep1_file))
 	 {
 	   fprintf (stderr, "\nERROR: %s DOES NOT EXIST[FATAL:%s]\n",  pep1_file, PROGRAM);
-	   exit(EXIT_FAILURE);
+	   myexit(EXIT_FAILURE);
 	 }
 	if (  pep2_file[0] && !check_file_exists ( pep2_file))
 	 {
 	   fprintf (stderr, "\nERROR: %s DOES NOT EXIST[FATAL:%s]\n",  pep2_file, PROGRAM);
-	   exit(EXIT_FAILURE);
+	   myexit(EXIT_FAILURE);
 	 }
 	
        if ( alignment1_file[0])sprintf ( seq_list[n_seq_file++], "A%s", alignment1_file);
@@ -694,6 +694,7 @@ if ( aln_compare==1)pep_compare=0;
 	   free_int ( aln_output_count, -1  );
 	 }
        }
+    return EXIT_SUCCESS;
     }   
 /************************************************************************************/
 /*                                                                                  */
@@ -1124,9 +1125,9 @@ fprintf ( stderr, "\n-pep (compare only the sequences");
 fprintf ( stderr, "\n-count");
 fprintf ( stderr, "\n-pep1 pep1_file");
 fprintf ( stderr, "\n-pep1 pep2_file");
-fprintf ( stderr, "\n-st  str_file st_format convertion");
+fprintf ( stderr, "\n-st  str_file st_format conversion");
 fprintf ( stderr, "\n   **st_format:  aln, pep");
-fprintf ( stderr, "\n   **convertion: 3d_ali,  conv abcZ #X");
+fprintf ( stderr, "\n   **conversion: 3d_ali,  conv abcZ #X");
 fprintf ( stderr, "\nNOTE: Several structures in a row are possible"); 
 
 fprintf ( stderr, "\n\n****DISTANCE MEASURE*****************");
@@ -1158,14 +1159,14 @@ fprintf ( stderr, "\n-output_aln_file      [stdout]");
 fprintf ( stderr, "\n-output_aln_format    [clustalw]");
 fprintf ( stderr, "\n-output_aln_modif     [lower]");
 fprintf ( stderr, "\n");
-exit (EXIT_SUCCESS);
+myexit (EXIT_SUCCESS);
 }
 
 /*********************************COPYRIGHT NOTICE**********************************/
 /* Centre National de la Recherche Scientifique (CNRS) */
 /*and */
 /*Cedric Notredame */
-/*Tue May 10 12:08:44     2005. */
+/*Fri Oct 26 17:03:04     2007. */
 /*All rights reserved.*/
 /*This file is part of T-COFFEE.*/
 /**/
