@@ -115,6 +115,7 @@ Dp_Result * make_fast_generic_dp_pair_wise (Alignment *A, int*ns, int **l_s,Dp_M
 
 Constraint_list* free_dp_model  (Dp_Model *D);
 Dp_Result * free_dp_result (Dp_Result *D );
+int ** ktup_dist_mat ( char **seq, int nseq, int ktup, char *type);
 int ** evaluate_diagonals_for_two_sequences ( char *seq1, char *seq2,int maximise,Constraint_list *CL,int ktup);
 int ** evaluate_diagonals ( Alignment *A, int *ns, int **l_s, Constraint_list *CL,int maximise,int n_groups, char **group_list,int ktup);
 int ** evaluate_segments_with_ktup  ( Alignment *A, int *ns, int **l_s, Constraint_list *CL,int maximise,int n_groups, char **group_list,int ktup);
@@ -147,7 +148,17 @@ int fasta_gotoh_pair_wise_sw (Alignment *A,int*ns, int **l_s,Constraint_list *CL
 int make_fasta_gotoh_pair_wise_sw (Alignment *A,int*ns, int **l_s,Constraint_list *CL, int *diag);
 
 /*pair wise aln implementations*/
+
+int idscore_pairseq (char *s1, char *s2, int gop, int gep, int **m, char *mode);
+int idscore_pair_wise (Alignment *A,int*ns, int **l_s,Constraint_list *CL);
 int gotoh_pair_wise    (Alignment *A,int*ns, int **l_s,Constraint_list *CL);
+int glocal_pair_wise ( Alignment *A, int *ns, int **l_s, Constraint_list *CL);
+int gotoh_pair_wise_lgp ( Alignment *A, int *ns, int **l_s, Constraint_list *CL);
+int test_pair_wise (Alignment *A, int *ns, int **l_s, Constraint_list *CL);
+
+int glocal2_pair_wise (Alignment *A,int*ns, int **ls,Constraint_list *CL);
+int subop1_pair_wise ( Alignment *A, int *ns, int **ls, Constraint_list *CL);
+int subop2_pair_wise ( Alignment *A, int *ns, int **ls, Constraint_list *CL);
 
   
       
@@ -218,7 +229,14 @@ Alignment * modify_domain ( Constraint_list *CL, Alignment *IN, int *start, int
 
 int * analyse_sequence ( Constraint_list *CL);
 
-
+/****************************************************************************/
+/*                                                                          */	
+/*                                                                          */	
+/*            Alignment Methods                                             */
+/*                                                                          */	
+/*                                                                          */	
+/****************************************************************************/
+Alignment * sorted_aln (Alignment *A, Constraint_list *CL);
 
 /******************************************************************/
 /*                   MAIN DRIVER                                  */
@@ -226,12 +244,12 @@ int * analyse_sequence ( Constraint_list *CL);
 /*                                                                */
 /******************************************************************/
 
-
-
 Constraint_list *profile2list     ( Job_TC *job,int nprf);
 Constraint_list *seq2list     (Job_TC *Job);
 Constraint_list *method2pw_cl (TC_method *M, Constraint_list *CL);
-int method_uses_structure( char *method);
+int method_uses_structure(TC_method *M);
+int method_uses_profile(TC_method *M);
+
 /******************************************************************/
 /*                   MULTIPLE ALIGNMENTS                          */
 /*                                                                */
@@ -240,6 +258,7 @@ int method_uses_structure( char *method);
 Alignment * compute_prrp_aln (Alignment *A, Constraint_list *CL);
 Alignment * compute_clustalw_aln (Alignment *A, Constraint_list *CL);
 Alignment * compute_tcoffee_aln_quick (Alignment *A, Constraint_list *CL);
+Alignment * seq2clustalw_aln (Sequence *S);
 Alignment * aln2clustalw_aln (Alignment *A, Constraint_list *CL);
 Alignment * realign_block ( Alignment *A, int col1, int col2, char *pg);
 /******************************************************************/
@@ -253,19 +272,18 @@ Constraint_list * align_coding_nucleotides (char *seq, char *method, char *weigh
 /*                                                                */
 /*                                                                */
 /******************************************************************/
-
+Constraint_list * seq_msa (TC_method *M , char *in_seq, Constraint_list *CL);
 
 Constraint_list *align_pdb_pair   (char *seq_in, char *dp_mode,char *evaluate_mode, char *file, Constraint_list *CL, Job_TC *job);
 Constraint_list * align_pdb_pair_2 (char *seq, Constraint_list *CL);
 
-Constraint_list * fugue_pair  ( char *seq, Constraint_list *CL);
-Constraint_list * fugue_pair2 (int s1, int s2, Constraint_list *CL);
-Constraint_list * sap_pair (char *seq, Constraint_list *CL);
+Constraint_list * pdb_pair  ( TC_method*M,char *seq, Constraint_list *CL);
+Constraint_list * profile_pair  ( TC_method*M,char *seq, Constraint_list *CL);
+Constraint_list * thread_pair  ( TC_method*M,char *seq, Constraint_list *CL);
+Constraint_list * thread_pair2 ( TC_method *M,int s1, int s2, Constraint_list *CL);
+Constraint_list * sap_pair (char *seq, char *weight, Constraint_list *CL);
 Constraint_list * lsqman_pair (char *seq, Constraint_list *CL);
 
-Constraint_list * thread_pair (char *seq, Constraint_list *CL);
-
-Constraint_list * sap_pair_rs (char *seq, Constraint_list *CL);
 
 /******************************************************************/
 /*                   GENERIC PAIRWISE METHODS                     */
@@ -278,20 +296,40 @@ Alignment * fast_pair      (Job_TC *job);
 void toggle_case_in_align_two_sequences(int value);
 Alignment * align_two_sequences ( char *seq1, char *seq2, char *matrix, int gop, int gep, char *align_mode);
 Alignment * align_two_aln ( Alignment *A1, Alignment  *A2, char *in_matrix, int gop, int gep, char *in_align_mode);
-NT_node ** make_tree ( Alignment *A,Constraint_list *CL,int gop, int gep,Sequence *S,  char *tree_file, char *tree_mode, int maximise);
+NT_node make_root_tree ( Alignment *A,Constraint_list *CL,int gop, int gep,Sequence *S,  char *tree_file,int maximise);
+NT_node ** make_tree ( Alignment *A,Constraint_list *CL,int gop, int gep,Sequence *S,  char *tree_file, int maximise);
 int ** get_pw_distances ( Alignment *A,Constraint_list *CL,int gop, int gep, char **out_seq, char **out_seq_name, int out_nseq, char *tree_file, char *tree_mode, int maximise);
-Alignment *build_progressive_nol_aln_with_seq_coor(Constraint_list *CL,int gop, int gep,Sequence *S, int **seq_coor, int nseq);
-Alignment *build_progressive_aln_with_seq_coor (Constraint_list*CL,int gop, int gep, Sequence *S, int **coor, int nseq);
-Alignment *build_progressive_aln(Alignment *A, Constraint_list *CL, int gop, int gep);
+Alignment *stack_progressive_nol_aln_with_seq_coor(Constraint_list *CL,int gop, int gep,Sequence *S, int **seq_coor, int nseq);
+Alignment *stack_progressive_aln_with_seq_coor (Constraint_list*CL,int gop, int gep, Sequence *S, int **coor, int nseq);
+Alignment *stack_progressive_aln(Alignment *A, Constraint_list *CL, int gop, int gep);
 Alignment *est_progressive_aln(Alignment *A, Constraint_list *CL, int gop, int gep);
 void analyse_seq ( Alignment *A, int s);
 
+char ** list_file2dpa_list_file (char **list_file, int *len,int maxnseq, Sequence *S);
+Alignment * seq2aln_group (Alignment *A, int T, Constraint_list *CL);
+
+Alignment *profile_aln (Alignment *A, Constraint_list *CL);
+Alignment * iterative_tree_aln (Alignment *A,int n, Constraint_list *CL);
+Alignment * iterative_aln ( Alignment*A, int nseq, Constraint_list *CL);
+Alignment * seq_aln ( Alignment*A, int nseq, Constraint_list *CL);
 Alignment *tsp_aln (Alignment *A, Constraint_list *iCL, Sequence *S);
 Alignment *iterate_aln ( Alignment*A, int nit, Constraint_list *CL);
 Alignment *realign_aln ( Alignment*A, Constraint_list *CL);
 Alignment *very_fast_aln (Alignment*A, int nseq, Constraint_list *CL);
+Alignment *simple_progressive_aln (Sequence *S, NT_node **T, Constraint_list *CL, char *mat);
+Alignment *frame_aln (Alignment *A, int n,Constraint_list *CL);
+Alignment *dpa_aln (Alignment *A, Constraint_list *CL);
+Alignment *new_dpa_aln (Alignment *A, Constraint_list *CL);
+Alignment * make_delayed_tree_aln (Alignment *A,int n, Constraint_list *CL);
+
+NT_node* delayed_tree_aln ( NT_node LT, NT_node RT, Alignment*A, int nseq, Constraint_list *CL);
+int  node2seq_list (NT_node P, int *ns, int *ls);
+Alignment* delayed_tree_aln1 ( NT_node P,Alignment*A,Constraint_list *CL, int threshold);
+Alignment* delayed_tree_aln2 ( NT_node P,Alignment*A,Constraint_list *CL, int threshold);
 
 NT_node* tree_aln ( NT_node LT, NT_node RT, Alignment*A, int nseq, Constraint_list *CL);
+NT_node* tree_realn ( NT_node LT, NT_node RT, Alignment*A, int nseq, Constraint_list *CL);
+
 int split_condition (int nseq, int score, Constraint_list *CL);
 
 int pair_wise (Alignment *A, int*ns, int **l_s,Constraint_list *CL );
@@ -320,6 +358,8 @@ int custom_pair_score_function7  (Constraint_list *CL, int s1, int r1, int s2, i
 int custom_pair_score_function8  (Constraint_list *CL, int s1, int r1, int s2, int r2);
 int custom_pair_score_function9  (Constraint_list *CL, int s1, int r1, int s2, int r2);
 int custom_pair_score_function10 (Constraint_list *CL, int s1, int r1, int s2, int r2);
+int apdb (int argc, char *argv[]);
+
 Constraint_list * set_constraint_list4align_pdb (Constraint_list *inCL,int seq, char *dp_mode, char *hasch_mode, char *param_file);
 int evaluate_ca_trace_sap2_bubble (Constraint_list *CL, int s1, int r1, int s2, int r2);
 int evaluate_ca_trace_nb (Constraint_list *CL, int s1, int s2, int r1, int r2);
@@ -348,14 +388,22 @@ float sap2_neighborhood_match (Constraint_list *CL, int s1, int s2, int r1, int
 /*         FUNCTIONS FOR COMPARING TWO NEIGHBORHOODS:END                                     */
 /*                                                                                           */
 /*********************************************************************************************/
-Alignment * analyse_pdb ( Alignment *A, Alignment *ST);
-float *** analyse_pdb_residues ( Alignment *A, Constraint_list *CL, Pdb_param *pdb_param);
+Alignment * analyse_pdb ( Alignment *A, Alignment *ST, char *filename);
+float **** analyse_pdb_residues ( Alignment *A, Constraint_list *CL, Pdb_param *pdb_param);
 
 float square_atom ( Atom *X);
 Atom* reframe_atom ( Atom *X, Atom*Y, Atom *Z, Atom *IN, Atom *R);
 Atom* add_atom ( Atom *A, Atom*B, Atom *R);
 Atom* diff_atom ( Atom *A, Atom*B, Atom *R);
 Atom * copy_atom ( Atom *A, Atom*R);
+/************************************************************************/
+/*                                                                      */
+/*                            NUSSINOV                                  */
+/*                                                                      */
+/************************************************************************/
+char *nussinov (char *S, int min_dist);
+char * rna_struc2rna_lib ( char *seq_name, char *seq, char *name);
+int display_rna_ss ( int pos, char *seq, char *struc, FILE *fp);
 int evaluate_aln_gibbs   ( Alignment *A, Constraint_list *CL);
 int evaluate_moca_domain ( Alignment *A, Constraint_list *CL);
 int moca_residue_pair_extended_list ( Constraint_list *CL, int s1, int r1, int s2, int r2);
@@ -404,15 +452,26 @@ double compute_X (double *n,int i,Mixture *D);
 Mixture *read_dirichlet ( char *name);
 int dirichlet_code( char aa);
 
+double lgamma2 ( double x);
+double lgamma_r(double x, int *signgamp);
 /*********************************************************************************************/
 /*                                                                                           */
 /*         FUNCTIONS FOR EVALUATING THE CONSISTENCY BETWEEN ALN AND CL                       */
 /*                                                                                           */
 /*********************************************************************************************/
 Alignment * main_coffee_evaluate_output          ( Alignment *IN,Constraint_list *CL, const char *mode );
+
+int sub_aln2sub_aln_score ( Alignment *IN,Constraint_list *CL, const char *mode, int *ns, int **ls);
+Alignment *  main_coffee_evaluate_output_sub_aln ( Alignment *IN,Constraint_list *CL, const char *mode, int *ns, int **ls);
+
 Alignment * categories_evaluate_output               ( Alignment *IN,Constraint_list *CL);
 Alignment * matrix_evaluate_output               ( Alignment *IN,Constraint_list *CL);
+Alignment * sar_evaluate_output ( Alignment *IN,Constraint_list *CL);
+Alignment * boxshade_evaluate_output ( Alignment *IN,Constraint_list *CL, int T);
+
 Alignment * fast_coffee_evaluate_output          ( Alignment *IN,Constraint_list *CL);
+
+int slow_coffee_evaluate_sub_aln         ( Alignment *IN,int *ns, int **ls, Constraint_list *CL);
 Alignment * slow_coffee_evaluate_output          ( Alignment *IN,Constraint_list *CL);
 Alignment * non_extended_t_coffee_evaluate_output( Alignment *IN,Constraint_list *CL);
 Alignment * heuristic_coffee_evaluate_output     ( Alignment *IN,Constraint_list *CL);
@@ -444,6 +503,12 @@ int evaluate_combined_matrix_score                 ( Constraint_list *CL, int s1
 int evaluate_physico_score                         ( Constraint_list *CL, int s1, int r1, int s2, int r2);
 int residue_pair_non_extended_list                 ( Constraint_list *CL, int s1, int r1, int s2, int r2);
 
+int residue_pair_extended_list4rna1                 ( Constraint_list *CL, int s1, int r1, int s2, int r2);
+int residue_pair_extended_list4rna2                 ( Constraint_list *CL, int s1, int r1, int s2, int r2);
+int residue_pair_extended_list4rna3                 ( Constraint_list *CL, int s1, int r1, int s2, int r2);
+int residue_pair_extended_list4rna4                 ( Constraint_list *CL, int s1, int r1, int s2, int r2);
+int residue_pair_extended_list4rna                 ( Constraint_list *CL, Constraint_list *R, int s1, int r1, int s2, int r2);
+
 int residue_pair_extended_list                     ( Constraint_list *CL, int s1, int r1, int s2, int r2);
 int residue_pair_extended_list_g_coffee_quadruplet ( Constraint_list *CL, int s1, int r1, int s2, int r2);
 int residue_pair_extended_list_g_coffee            ( Constraint_list *CL, int s1, int r1, int s2, int r2);
@@ -468,13 +533,22 @@ int get_dp_cost_quadruplet      ( Alignment *A, int**pos1, int ns1, int*list1, i
       
 
 int very_fast_get_dp_cost       ( Alignment *A, int**pos1, int ns1, int*list1, int col1, int**pos2, int ns2, int*list2, int col2, Constraint_list *CL);
-int super_fast_get_dp_cost  ( Alignment *A, int**pos1, int ns1, int*list1, int col1, int**pos2, int ns2, int*list2, int col2, Constraint_list *CL);
-int super_fast_get_dp_cost2 ( Alignment *A, int**pos1, int ns1, int*list1, int col1, int**pos2, int ns2, int*list2, int col2, Constraint_list *CL);
+
+int cw_profile_get_dp_cost ( Alignment *A, int**pos1, int ns1, int*list1, int col1, int**pos2, int ns2, int*list2, int col2, Constraint_list *CL);
+int cw_profile_get_dp_cost_window ( Alignment *A, int**pos1, int ns1, int*list1, int col1, int**pos2, int ns2, int*list2, int col2, Constraint_list *CL);
+
+int consensus_get_dp_cost ( Alignment *A, int**pos1, int ns1, int*list1, int col1, int**pos2, int ns2, int*list2, int col2, Constraint_list *CL);
 
 int fast_get_dp_cost            ( Alignment *A, int**pos1, int ns1, int*list1, int col1, int**pos2, int ns2, int*list2, int col2, Constraint_list *CL);
 int fast_get_dp_cost_2          ( Alignment *A, int**pos1, int ns1, int*list1, int col1, int**pos2, int ns2, int*list2, int col2, Constraint_list *CL);
+int fast_get_dp_cost_3          ( Alignment *A, int**pos1, int ns1, int*list1, int col1, int**pos2, int ns2, int*list2, int col2, Constraint_list *CL);
+
 int fast_get_dp_cost_quadruplet ( Alignment *A, int**pos1, int ns1, int*list1, int col1, int**pos2, int ns2, int*list2, int col2, Constraint_list *CL);
 
+int check_fast_profile_mode    (Alignment *A, int ns1,int *list1,int ns2, int *list2, Constraint_list *CL);
+int check_fast_mode    (Alignment *A, int ns1,int *list1,int ns2, int *list2, Constraint_list *CL);
+
+
 int slow_get_dp_cost            ( Alignment *A, int**pos1, int ns1, int*list1, int col1, int**pos2, int ns2, int*list2, int col2, Constraint_list *CL);
 int slow_get_dp_cost_test       ( Alignment *A, int**pos1, int ns1, int*list1, int col1, int**pos2, int ns2, int*list2, int col2, Constraint_list *CL);
 
@@ -499,12 +573,19 @@ float compare_two_mat (char  *mat1,char*mat2, int n, int len,char *alp);
 float compare_two_mat_array (int  **matrix1,int **matrix2, int n, int len,char *alp);
 int ** rescale_two_mat (char  *mat1,char*mat2, int n, int len,char *alp);
 int ** rescale_matrix ( int **mat, float lambda, char *alp);
+int **mat2inverted_mat (int **matrix, char *alp);
 void output_matrix_header ( char *name, int **matrix, char *alp);
 float evaluate_random_match2 (int **matrix, int n, int len,char *alp);
 float measure_lambda2(char  *mat1,char*mat2, int n, int len,char *alp);
 float measure_lambda(char  *mat1,char*mat2, int n, int len,char *alp);
 Constraint_list * choose_extension_mode ( char *extend_mode, Constraint_list *CL);
 int ** combine_two_matrices ( int **mat1, int **mat2);
+/*********************************************************************************************/
+/*                                                                                           */
+/*         OFF THE SHELVES EVALUATION                                              */
+/*                                                                                           */
+/*********************************************************************************************/
+float  sum_pair ( Alignment*A,char *mat_name, int gap_op, int gap_ext);
 int ** show_pair(int istart, int iend, int jstart, int jend, int *seqlen_array, char **seq_array, int dna_ktup, int dna_window, int dna_wind_gap, int dna_signif,int prot_ktup, int prot_window,int prot_wind_gap,int prot_signif, int nseqs,int dnaflag, int max_aa, int max_aln_length  );
 /*By convention, 0: START, 1: END*/
 
@@ -542,7 +623,7 @@ typedef double (*Generic_em_func)(struct Hmm*, struct HmmState*, int);
 struct HmmState
 {
 char name[100];
-char state;
+int state;
 int DJ;
 int DI;
 
@@ -665,14 +746,18 @@ Hmm* define_simple_model(Constraint_list *CL);
 
 Hmm * bound_hmm ( Hmm *H);
 Sequence *pavie_seq2random_seq (Sequence *S, char *subst); 
-float **pavie_seq2pavie_aln(Sequence *S,char *mat, char *mode);
+double **pavie_seq2pavie_aln(Sequence *S,char *mat, char *mode);
+Alignment *pavie_seq2pavie_sort ( Sequence *S, char *mat, char *mode);
+Alignment* pavie_seq2pavie_msa  ( Sequence *S, char *mat, char *mode);
+NT_node pavie_seq2pavie_tree ( Sequence *S, char *mat, char *mode);
 int **pavie_seq2trained_pavie_mat(Sequence *S, char *param);
 int pavie_pair_wise (Alignment *A,int *ns, int **l_s,Constraint_list *CL );
+Sequence *pavie_seq2noisy_seq (Sequence *S, int val,char *subst); 
 /*********************************COPYRIGHT NOTICE**********************************/
 /* Centre National de la Recherche Scientifique (CNRS) */
 /*and */
 /*Cedric Notredame */
-/*Tue May 10 12:08:44     2005. */
+/*Fri Oct 26 17:03:04     2007. */
 /*All rights reserved.*/
 /*This file is part of T-COFFEE.*/
 /**/
