@@ -42,16 +42,109 @@ double emitPairsDefault[20][20] = {
   {0.00131840f, 0.00099430f, 0.00074960f, 0.00066005f, 0.00036626f, 0.00070192f, 0.00092548f, 0.00089301f, 0.00131038f, 0.00127857f, 0.00219713f, 0.00100817f, 0.00054105f, 0.00368739f, 0.00047608f, 0.00102648f, 0.00094759f, 0.00069226f, 0.00999315f, 0.0f},
   {0.00533241f, 0.00169359f, 0.00136609f, 0.00127915f, 0.00119152f, 0.00132844f, 0.00178697f, 0.00194579f, 0.00071553f, 0.01117956f, 0.00914460f, 0.00210897f, 0.00197461f, 0.00256159f, 0.00135781f, 0.00241601f, 0.00343452f, 0.00038538f, 0.00148001f, 0.02075171f}
 };
-static int IncompatibleMatState(MatState*S, MatState*E);
+
 static void DisplayMatState ( MatState *S, char *s);
-static void filter_matrix (char *s1,char *s2, double **M, double T);
-static double*** probcons_pair_wise (char *seq1, char *seq2, int MODE);
-static void display_state_matrix2 ( MatState ****M, Hmm *H, int l1, int l2, int state, char *string);
-static void display_state_matrix ( double **M, Hmm *h, int l1, int l2, char *string);
+
+
+
+
+
 
 void check_viterbiL ( Alignment *A,int *ns, int **ls, Constraint_list *CL);
 MatState *viterbi2path2 ( double ***Sc, int ***St, Hmm *H, MatState *S, MatState *E);
 void testfunc ( MatState *S, char *s);
+
+#ifdef IN_PGROGRESS
+/*********************************************************************************/
+/*                                                                               */
+/*                                                                               */
+/*                     MSA Analyzer                                              */
+/*                                                                               */
+/*                                                                               */
+/*********************************************************************************/
+Alignment * analyze_alignment ( Alignment *A)
+{
+  evaluate_alignment (A);
+  H=define_msa_model (-100);
+  M=seq_viterbi_hmm (A->seq_al[0], H);
+  path=seq_viterbi2path ( seq, H, M);
+}
+    
+
+Hmm* define_msa_model(double penalty)
+{
+  Hmm *H;
+  double freeT=0;
+  int n=0;
+  HmmState *S;
+  
+  
+  H=declare_hmm(2);
+  H->freeT=freeT=0;
+ 
+  H->forbiden=FORBIDEN;
+  H->start=START_STATE;
+  H->end=END_STATE;
+ 
+  /*define START*/
+  S=H->S[n];  
+  sprintf (S->name, "START"); S->state=n;
+  
+  S->DI=0;
+  S->DJ=0;
+  S->em=freeT;
+ 
+  sprintf ( (S->T[S->nT])->name, "C") ;(S->T[S->nT])->tr=freeT  ;S->nT++;
+  sprintf ( (S->T[S->nT])->name, "W");(S->T[S->nT])->tr=freeT  ;S->nT++;
+  n++;
+  /*define END*/
+  S=H->S[n];  
+  sprintf (S->name, "END"); S->state=n;
+  S->DI=0;
+  S->DJ=0;
+  S->em=freeT;
+  n++;
+
+  /*define Correct*/
+  S=H->S[n];  
+  sprintf (S->name, "C"); S->state=n;
+  S->DI=1;
+  S->DJ=0;
+  S->em=H->forbiden;
+  S->em_func=em_correct_msa;
+  
+  sprintf ( (S->T[S->nT])->name, "C") ;(S->T[S->nT])->tr=freeT;S->nT++;
+  sprintf ( (S->T[S->nT])->name, "W");(S->T[S->nT])->tr=penalty  ;S->nT++;
+  sprintf ( (S->T[S->nT])->name, "END");   (S->T[S->nT])->tr=freeT;S->nT++;
+  n++;
+  
+  /*define Wrong*/
+  S=H->S[n];  
+  sprintf (S->name, "INSERT"); S->state=n;
+  S->DI=1;
+  S->DJ=0;
+  S->em=H->forbiden;
+  S->em_func=em_wrong_msa;
+  sprintf ( (S->T[S->nT])->name, "C") ; (S->T[S->nT])->tr=penalty;S->nT++;
+  sprintf ( (S->T[S->nT])->name, "W"); (S->T[S->nT])->tr=freeT;S->nT++;
+  sprintf ( (S->T[S->nT])->name, "END");    (S->T[S->nT])->tr=-gop;S->nT++;
+  n++;
+  
+  /*define LInsert*/
+  S=H->S[n];  
+  sprintf (S->name, "LINSERT"); S->state=n;
+  S->DI=1;
+  S->DJ=0;
+  S->em=lgep;
+  
+  sprintf ( (S->T[S->nT])->name, "INSERT")  ;(S->T[S->nT])->tr=freeT;S->nT++;
+  sprintf ( (S->T[S->nT])->name, "LINSERT");(S->T[S->nT])->tr=freeT;S->nT++;
+  n++;
+  
+  H=bound_hmm ( H);
+  return H;
+}
+#endif
 /*********************************************************************************/
 /*                                                                               */
 /*                                                                               */
@@ -135,7 +228,7 @@ char **produce_state_name (int nstate,char **list, char *model_name, Hmm* H)
       if ( a==H->start)sprintf ( new_list[a], "START");
       else if ( a==H->end)sprintf ( new_list[a], "END");
       else if ( list==NULL){sprintf ( new_list[a], "%c%s", 'a'+b, (model_name)?model_name:"");b++;}
-      else {sprintf ( new_list[a], "%c%s", list[c], (model_name)?model_name:"");c++;}
+      else {sprintf ( new_list[a], "%c%s", list[a][c], (model_name)?model_name:"");c++;}
     }
   return new_list;
 }
@@ -144,7 +237,8 @@ int seq_viterbi_pair_wise (Alignment *A,int*ns, int **ls,Constraint_list *CL)
 {
   ungap(A->seq_al[0]);
   analyze_sequence (A->seq_al[0], NULL);
-  exit (0);
+  myexit (EXIT_FAILURE);
+  return 1;
 }
 double analyze_sequence ( char *seq, Hmm *H)
 {
@@ -169,7 +263,7 @@ double** seq_viterbi_hmm (char *seq, Hmm *H)
   /*Given a model H and a sequence seq*/
   double **M;
   double e, v, max;
-  int a, i,pi, bestk,j,pj, s, k, l1, l2;
+  int i,pi, bestk, s, k, l1;
   HmmState *S1, *S2;
   
   
@@ -189,7 +283,7 @@ double** seq_viterbi_hmm (char *seq, Hmm *H)
 	  if ( pi<0){M[s][i]=H->forbiden;}/*Boundary*/
 	  else 
 	    {
-	      if (pi==0) {max=H->T[H->start][s];bestk=H->start;}/*Start*/
+	      if (pi==0) {max=H->T[(int)H->start][s];bestk=H->start;}/*Start*/
 	      else
 		{
 		  for (k=1; k<=H->fromM[S1->state][0]; k++)
@@ -200,7 +294,7 @@ double** seq_viterbi_hmm (char *seq, Hmm *H)
 		      if ( v!=H->forbiden && (max==H->forbiden || v>max)){max=v;bestk=S2->state;}
 		    }
 		} 	
-	      if (S1->em2)e=S1->em2[seq[pi]];
+	      if (S1->em2)e=S1->em2[(int)seq[pi]];
 	      else e=S1->em_func2(H,S1, (int)seq[pi]);
 	      
 	      e=hmm_add (e,max);
@@ -522,7 +616,7 @@ MatState* RviterbiL_hmm (Alignment *A,int *ns, int **ls, Hmm *H, Constraint_list
 	{
 	  DisplayHmm (H);
 	  fprintf ( stderr, "\nERROR: The Requested Model (Cf Above) Cannot Produce the Pair-Alignment\nYou must allow extra possible transitions\n[FATAL:%s]", PROGRAM);
-	  exit ( EXIT_FAILURE);
+	  myexit ( EXIT_FAILURE);
 	}
   E=S->n;
   
@@ -544,19 +638,19 @@ MatState* viterbiL_hmm (Alignment *A,int *ns, int **ls, Hmm *H, Constraint_list
 {
   int current, previous,row, prow;
   double v;
-  int a,b,i,j,pi,pj, s, k;
+  int a,i,j,pi,pj, s, k;
   int start_i, start_j, end_i, end_j, l1, l2;
   HmmState *S1, *S2;
   MatState *CC, *PCC,*tS, *tE, *mark=NULL;
   int midpoint;
-  int *jlist;
-  static int **main_jlist;
+
+
   static MatState ***M;
-  static int *toclean;
-  int ntoclean;
+
+
   static int LenJ, LenI;
   int MaxDelta=50, DeltaI, DeltaJ;
-  int mode;
+
   
 
   DisplayMatState (S, "\n\tS");
@@ -598,7 +692,7 @@ MatState* viterbiL_hmm (Alignment *A,int *ns, int **ls, Hmm *H, Constraint_list
       row=current;
       for ( j=start_j; j<=end_j; j++)
 	{
-	  DeltaJ=FABS((j-i)<MaxDelta)?1:MaxDelta+1; /*Keep Pointers on a DealtaMax Band around the main diagonal*/
+	  DeltaJ=((FABS(j-i))<MaxDelta)?1:MaxDelta+1; /*Keep Pointers on a DealtaMax Band around the main diagonal*/
 	  for ( s=H->nS-1;s>=0; s--)
 	    {
 	      S1=H->S[s];pi=i-S1->DI;prow=S1->DI;pj=j-S1->DJ;
@@ -828,7 +922,7 @@ MatState* RviterbiD_hmm (Alignment *A,int *ns, int **ls, Hmm *H, Constraint_list
 	{
 	  DisplayHmm (H);
 	  fprintf ( stderr, "\nERROR: The Requested Model (Cf Above) Cannot Produce the Pair-Alignment\nYou must allow extra possible transitions\n[FATAL:%s]", PROGRAM);
-	  exit ( EXIT_FAILURE);
+	  myexit ( EXIT_FAILURE);
 	}
   E=S->n;
   
@@ -858,7 +952,7 @@ MatState* viterbiD_hmm (Alignment *A,int *ns, int **ls, Hmm *H, Constraint_list
   
   int dj;
   int dc;
-  int *jlist;
+  int *jlist=NULL;
   static int **main_jlist;
   static MatState ***M;
   static int *toclean;
@@ -920,7 +1014,7 @@ MatState* viterbiD_hmm (Alignment *A,int *ns, int **ls, Hmm *H, Constraint_list
      
       for ( dj=1; dj<=jlist[0]; dj++)
 	{
-	  DeltaJ=FABS((dj-i)<MaxDelta)?1:MaxDelta+1; /*Keep Pointers on a 2*DeltaMax Band around the main diagonal*/
+	  DeltaJ=((FABS(dj-i))<MaxDelta)?1:MaxDelta+1; /*Keep Pointers on a 2*DeltaMax Band around the main diagonal*/
 
 	  dc=(mode==DIAGONALS && dj!=1)?i:0;/*Make sure the diagonal Mode uses the sides of the array*/
 	  j=jlist[dj]+dc;
@@ -1125,7 +1219,7 @@ double lu_RviterbiDGL_hmm (Alignment *A,int *ns, int **ls, Hmm *H, Constraint_li
   
   
   
-  if ( S==NULL | E==NULL || S->sc==H->forbiden) return H->forbiden;
+  if ( S==NULL || E==NULL || S->sc==H->forbiden) return H->forbiden;
   else if ( S->st==H->start) return 0;
   else if ( E->st==H->end) return S->sc;
   else if (min==0)
@@ -1178,7 +1272,7 @@ MatState* RviterbiDGL_hmm (Alignment *A,int *ns, int **ls, Hmm *H, Constraint_li
 	{
 	  DisplayHmm (H);
 	  fprintf ( stderr, "\nERROR: The Requested Model (Cf Above) Cannot Produce the Pair-Alignment\nYou must allow extra possible transitions\n[FATAL:%s]", PROGRAM);
-	  exit ( EXIT_FAILURE);
+	  myexit ( EXIT_FAILURE);
 	}
   E=S->n;
   
@@ -1201,7 +1295,7 @@ MatState* viterbiDGL_hmm (Alignment *A,int *ns, int **ls, Hmm *H, Constraint_lis
 {
   int current, previous,row, prow;
   double v;
-  int a,b,i,j,pi,pj, s, k;
+  int a,i,j,pi,pj, s, k;
   int start_i, start_j, end_i, end_j, l1, l2;
   HmmState *S1, *S2;
   MatState *CC, *PCC,*tS, *tE,*bestE,*bestS, *mark=NULL;
@@ -1209,7 +1303,7 @@ MatState* viterbiDGL_hmm (Alignment *A,int *ns, int **ls, Hmm *H, Constraint_lis
   
   int dj;
   int dc;
-  int *jlist;
+  int *jlist=NULL;
   static int **main_jlist;
   static MatState ***M;
   static int *toclean;
@@ -1217,7 +1311,7 @@ MatState* viterbiDGL_hmm (Alignment *A,int *ns, int **ls, Hmm *H, Constraint_lis
   static int LenJ, LenI;
   int MaxDelta=50, DeltaI, DeltaJ;
   int mode;
-  int debug=1;
+
   
 
   DisplayMatState (S, "\n\tS");
@@ -1274,7 +1368,7 @@ MatState* viterbiDGL_hmm (Alignment *A,int *ns, int **ls, Hmm *H, Constraint_lis
      
       for ( dj=1; dj<=jlist[0]; dj++)
 	{
-	  DeltaJ=FABS((dj-i)<MaxDelta)?1:MaxDelta+1; /*Keep Pointers on a 2*DeltaMax Band around the main diagonal*/
+	  DeltaJ=(FABS(dj-i)<MaxDelta)?1:MaxDelta+1; /*Keep Pointers on a 2*DeltaMax Band around the main diagonal*/
 
 	  dc=(mode==DIAGONALS && dj!=1)?i:0;/*Make sure the diagonal Mode uses the sides of the array*/
 	  j=jlist[dj]+dc;
@@ -1743,22 +1837,6 @@ MatState* CopyMatState ( MatState*I, MatState*O)
 /*                                                                               */
 /*                                                                               */
 /*********************************************************************************/
-int IncompatibleMatState(MatState*S, MatState*E)
-{
-  int di, dj;
-  if ( S==NULL || E==NULL) return 0;
-  di=E->i-S->i;    
-  dj=E->j-S->j;
-  
-  if ( S->st ==1 || E->st==1)return 0;
-  else if ( di==0 && dj==0)return 0;
-  else if (di==0 && dj>0)return 0;
-  else if ( di>0 && dj==0) return 0;
-  else if ( di==1 && dj==1)return 0;
-
-
-  return 1;
-}
 int MaxDeltaMatState (MatState*S, MatState*E)
 {
   if ( !S || !E) return -1;
@@ -1897,9 +1975,9 @@ Hmm* define_mnm_model(Constraint_list *CL)
 {
   Hmm *H;
   double gop=20;
-  double gep=0;
-  double lgop=0;
-  double lgep=0;
+
+
+
   double freeT=0;
   int n=0;
   HmmState *S;
@@ -2210,7 +2288,7 @@ Hmm* define_two_mat_model(Constraint_list *CL)
   if ( n!=H->nS)
     {
       fprintf ( stderr, "\nERROR in HMM definition [FATAL:%s]", PROGRAM);
-      exit (0);
+      myexit (EXIT_FAILURE);
     }
   
   H=bound_hmm ( H);
@@ -2229,46 +2307,37 @@ void testfunc ( MatState *S, char *s)
   fprintf ( stderr, "\n");
 }
   
-void display_state_matrix2 ( MatState ****M,Hmm*H, int l1, int l2, int s, char *string)
-{
-  int a, b;
 
-  fprintf (stderr, "\nMAtrix %s, %d %d", string, l1, l2);
-  for ( a=0; a<=l1; a++)
-    {
-      fprintf ( stderr, "\n");
-      for ( b=0; b<=l2; b++)
-	{
-	if ( M[a][b][s]->sc==H->forbiden) fprintf ( stderr, " %3c", '*');
-	else fprintf ( stderr, " %3d", (int)M[a][b][s]->sc);
-	}
-    }
-  fprintf (stderr, "\nMAtrix %s, %d %d (previous state)", string, l1, l2);
-  for ( a=0; a<=l1; a++)
-    {
-      fprintf ( stderr, "\n");
-      for ( b=0; b<=l2; b++)
-	{
-	if ( M[a][b][s]->sc==H->forbiden) fprintf ( stderr, " %3c", '*');
-	else fprintf ( stderr, " %3d", (int)M[a][b][s]->pst);
-	}
-    }
-}
-void display_state_matrix ( double **M, Hmm *H, int l1, int l2, char *string)
-{
-  int a, b;
 
-  fprintf (stderr, "\nMAtrix %s, %d %d", string, l1, l2);
-  for ( a=0; a<=l1; a++)
-    {
-      fprintf ( stderr, "\n");
-      for ( b=0; b<=l2; b++)
-	{
-	if ( M[a][b]==H->forbiden) fprintf ( stderr, " %3c", '*');
-	else fprintf ( stderr, " %3d", (int)M[a][b]);
-	}
-    }
-}
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
 #ifdef BACKHERE
 	  
@@ -2444,7 +2513,7 @@ MatState* viterbiL_hmm (Alignment *A,int *ns, int **ls, Hmm *H, Constraint_list
 /* Centre National de la Recherche Scientifique (CNRS) */
 /*and */
 /*Cedric Notredame */
-/*Tue May 10 12:08:44     2005. */
+/*Fri Oct 26 17:03:04     2007. */
 /*All rights reserved.*/
 /*This file is part of T-COFFEE.*/
 /**/
