@@ -201,7 +201,7 @@ int string2key (char *s, Char_node *n)
     }
   else
     {
-      return string2key(s+1, (n->c[s[0]])?(n->c[s[0]]):(n->c[s[0]]=declare_char_node (DECLARE)));
+      return string2key(s+1, (n->c[(int)s[0]])?(n->c[(int)s[0]]):(n->c[(int)s[0]]=declare_char_node (DECLARE)));
     }
   return 0;
 }
@@ -263,7 +263,7 @@ Char_node * declare_char_node (int action)
 /* Centre National de la Recherche Scientifique (CNRS) */
 /*and */
 /*Cedric Notredame */
-/*Tue May 10 12:08:44     2005. */
+/*Fri Oct 26 17:03:04     2007. */
 /*All rights reserved.*/
 /*This file is part of T-COFFEE.*/
 /**/
