@@ -1,7 +1,8 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <ctype.h>
-
+#include <string.h>
+#include <ctype.h>
 #include "io_lib_header.h"
 #include "util_lib_header.h"
 #include "define_header.h"
@@ -48,13 +49,20 @@ char *    get_string_type   (char *S)
 	int a, l;
 	int protein=0,  dna=0, tot=0;
 	char *type;
+	static char *ltype;
+	static int warning;
 	
-
+	if ( !ltype)
+	  declare_name(ltype);
 	
 	declare_name(type);
 	l=(int)strlen (S);
 	
-	
+	if (l==0)
+	  {
+	    sprintf ( type, "UNKNOWN");
+	    return type;
+	  }
 	
 	for ( a=0; a<l; a++)     
 	        {
@@ -66,20 +74,39 @@ char *    get_string_type   (char *S)
 			}
 		}
 
-
+	protein=(protein*100)/tot;
+	dna=(dna*100)/tot;
 	
-	if (protein>0){sprintf (type, "PROTEIN");}
-	else if ( dna>0 && tot>10){sprintf (type, "DNA");}
-	else
-	  {sprintf ( type, "UNKNOWN");}
-
+	if ( l<20 && warning==0)
+	  {
+	    /*add_warning ( stderr, "WARNING: short sequences, use -type=DNA or PROTEIN");*/
+	    warning=1;
+	  }
+	if ( l<20 && ltype && ltype[0])
+	  {
+	    if (dna==100)
+	      {
+		sprintf ( type, "%s", ltype);
+		ltype[0]='\0';
+	      }
+	    else
+	      sprintf ( type, "PROTEIN");
+	    
+	  }
+	else if ( dna>98)sprintf ( type, "DNA");
+	else sprintf ( type, "PROTEIN");
+	
+	sprintf ( ltype, "%s", type);
 	return type;
 	}
 
 Alignment* get_aln_type (Alignment *A)
         {
-
-	  if ( A->S!=NULL && (A->S)->type[0]!='\0')
+	  if ( !A) return A;
+	  
+	  if ( A->S && !(A->S)->type)(A->S)->type=vcalloc (30, sizeof (char));
+	  
+	  if ( A->S && (A->S)->type[0]!='\0')
 	    {
 	    ;
 	    }
@@ -94,8 +121,54 @@ Alignment* get_aln_type (Alignment *A)
 	      }
 	  return A;
 	}
-  
+char *unset_seq_type ()
+{
+  return set_seq_type (UNSET, NULL);
+}
+char *store_seq_type (char *val)
+{
+  return set_seq_type (SET, val);
+}
+char *retrieve_seq_type ()
+{
+  return set_seq_type (GET,NULL);
+}
+char *set_seq_type (int mode, char *val)
+{
+  static char type[100];
+  if (mode==SET)
+    {
+      if (!val)printf_exit (EXIT_FAILURE, stderr, "Error: sequence type unset in io_func.c:set_seq_type");
+      sprintf ( type,"%s", val);
+    }
+  else if ( mode==GET)
+    {
+      ;
+    }
+  else if ( mode==UNSET)
+    {
+      type[0]='\0';
+    }
+  else
+    {
+      printf_exit (EXIT_FAILURE, stderr, "Error: unknown mode in function io_func.c:set_seq_type, use SET, GET or UNSET");
+    }
+  return type;
+}
 Sequence *get_sequence_type (Sequence *S)
+        {
+	int a;
+	char *buf;
+	int tot;
+
+	for ( tot=0,a=0; a< S->nseq; a++)tot+=strlen (S->seq[a]);
+	buf=vcalloc (tot+1, sizeof (char));
+	for ( a=0; a<S->nseq; a++)strcat (buf, S->seq[a]);
+	sprintf ( S->type, "%s", get_string_type(buf));
+	vfree (buf);
+	return S;
+	}
+Sequence *get_sequence_type_old (Sequence *S)
         {
 	int a;
 	int protein=0,  dna=0;
@@ -113,8 +186,8 @@ Sequence *get_sequence_type (Sequence *S)
 	
 
 	if ( protein==0 && dna==0);
-	else if ( protein==0 )sprintf ( S->type, "DNA");
-	else if ( dna==0)sprintf ( S->type, "PROTEIN");
+	else if ( dna>protein )sprintf ( S->type, "DNA");
+	else if ( protein>dna)sprintf ( S->type, "PROTEIN");
 	else sprintf ( S->type, "PROTEIN_DNA");
 	return S;
 	}
@@ -127,12 +200,7 @@ void get_sequence (char *seq_file,int *NSEQ, char ***SEQ, char ***SN, int **sl,
 	int nseq;
 
 	int **SL;
-	
-	
-
-	
-	
-	
+		
 	nseq=NSEQ[0]= readseqs ( seq_file,  SEQ, SN, &SL);
 	sl[0]=vcalloc ( nseq, sizeof (int));
 	
@@ -161,7 +229,7 @@ int ** get_matrix   ( char *name, char *format)
        else
            {
 	   fprintf ( stderr, "\nError:\nUnknowm Format %s for Matrix %s[FATAL]", format, name);
-	   exit (1);
+	   myexit (EXIT_FAILURE);
 	   }
        return NULL;
        }
@@ -169,7 +237,7 @@ void display_matrix (int **mat);
 int ** read_matrice (char *mat_name_in)
 	{
 	int a,b,c, l;
-	int max, min;
+
 	char AA[]="abcdefghiklmnpqrstvwxyz";
         FILE *fp;
 	int **matrice;
@@ -187,9 +255,10 @@ int ** read_matrice (char *mat_name_in)
 	else if (strm3 (mat_name_in, "id", "ID", "idmat"))sprintf ( mat_name, "idmat");
 	else sprintf ( mat_name, "%s", mat_name_in);
 	
-
+	/*Read Header Matrices*/
 	if (strm(mat_name, "pam250mt"))vector=pam250mt;
 	else if (strm(mat_name, "idmat"))vector=idmat;
+	else if (strm(mat_name, "dna_idmat"))vector=idmat;
 	else if (strm(mat_name, "est_idmat"))vector=est_idmat;
 	else if (strm(mat_name, "md_350mt"))vector=md_350mt;
 	else if (strm(mat_name, "md_250mt"))vector=md_250mt;
@@ -201,6 +270,9 @@ int ** read_matrice (char *mat_name_in)
 	
 	else if (strm(mat_name, "blosum80mt" ))vector=blosum80mt;
 	else if (strm(mat_name, "blosum62mt" ))vector=blosum62mt;
+	else if (strm(mat_name, "exon2mt" ))vector=blosum62mt;
+	else if (strm(mat_name, "blosum62mt3" ))vector=blosum62mt3;
+	
 	else if (strm(mat_name, "blosum62mt2" ))vector=blosum62mt2;
 	else if (strm(mat_name, "blosum55mt" ))vector=blosum55mt;
 	else if (strm(mat_name, "blosum50mt" ))vector=blosum50mt;
@@ -228,18 +300,22 @@ int ** read_matrice (char *mat_name_in)
 	else if (strm(mat_name, "ralpha_mat" ))vector=ralpha_mat;
 	else if (strm(mat_name, "rcoil_mat" ))vector=rcoil_mat;
 	
+
+	/*Header Matrices*/
 	if(vector)
 	  {
 	    matrice=declare_int ( 256, 256);
 	    for (a=0; a<l; ++a)
 		{
 		 for (b=0;b<=a;++b)
-				{matrice[a][b]=matrice[b][a]=(vector[(a*a+a)/2+b]);
-				
-				}
+		   {
+		     matrice[a][b]=matrice[b][a]=(vector[(a*a+a)/2+b]);
+		   }
 		}
-	
 	  }
+
+	  /*Hard coded Matrices*/
+	
 	else if ( strm (mat_name, "exon_mt"))
 	  {
 	    matrice=declare_int ( 256, 256);
@@ -255,7 +331,18 @@ int ** read_matrice (char *mat_name_in)
 	      }
 	    
 	  }
-	    
+	else if ( strm ( mat_name, "tdamat"))
+	  {
+	    int x, y;
+	    matrice=declare_int ( 256, 256);
+	    sprintf ( AA, "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ");
+	    l=strlen (AA);
+	    for ( x=0; x<l; x++)
+	      for ( y=0; y<l; y++)matrice[x][y]=-10;
+	    for ( x=0; x<l; x++) {matrice[x][x]=0; matrice[x][GAP_CODE]=-3;}
+	    return matrice;
+	  }
+	/*Blast Matrices*/
 	else if ( check_file_exists(mat_name) && is_blast_matrix (mat_name))
 	  {
 	    
@@ -269,8 +356,9 @@ int ** read_matrice (char *mat_name_in)
 	    return matrice;
 	  }
 	
-	else if ( check_file_exists(mat_name))
+	else if ( check_file_exists(mat_name) && is_clustalw_matrix (mat_name))
 		{
+		  
 		  fp=vfopen ( mat_name, "r");
 		  while ( (c=fgetc (fp))!='$' && c!=EOF);
 		  if ( c==EOF){vfclose (fp);return NULL;};
@@ -280,7 +368,7 @@ int ** read_matrice (char *mat_name_in)
 		  for ( a=0; a<l; a++)
 		    {
 		      for ( b=0; b<=a; b++)
-			if (fscanf ( fp, "%d,", &matrice[a][b])==0){vfclose (fp);free_int (matrice, -1);free_int (matrix2, -1);return NULL;};
+			if (fscanf ( fp, "%d,", &matrice[a][b])==0){vfclose (fp);free_int (matrice, -1);return NULL;};
 		      fscanf ( fp, "\n");
 		    }		
 		  fclose ( fp);
@@ -298,6 +386,20 @@ int ** read_matrice (char *mat_name_in)
 			matrix2[tolower(AA[b])-'A'][toupper(AA[c])-'A']=matrice[b][c];
 			matrix2[tolower(AA[b])-'A'][tolower(AA[c])-'A']=matrice[b][c];
 			}
+	if ( strm ( "exon2mt", mat_name))
+	  {
+	    char EE[4];
+	    int v=(getenv ("EXONVALUE_4_TCOFFEE"))?atoi("EXONVALUE_4_TCOFFEE"):100;
+	    sprintf (EE, "obj");
+	    for (a=0; a<strlen (EE); a++)
+	      {
+		matrix2[toupper(EE[a])-'A'][toupper(EE[a])-'A']=v;
+		matrix2[tolower(EE[a])-'A'][toupper(EE[a])-'A']=v;
+		matrix2[toupper(EE[a])-'A'][tolower(EE[a])-'A']=v;
+		matrix2[tolower(EE[a])-'A'][tolower(EE[a])-'A']=v;
+	      }
+	  }
+	
 	free_int (matrice, -1);
 	return (matrix2);
 
@@ -460,7 +562,7 @@ void get_rgb_values ( int val, Color *C)
 
 
      /*Conversions*/
-       if ( val==10)val--;
+     if ( val==10)val--;
        
      r=&C->r;
      g=&C->g;
@@ -506,103 +608,9 @@ void get_rgb_values ( int val, Color *C)
 	}
      return;
 
-     if ( val==0)
-        {
-	sprintf ( C->html_color, "%s", html_code[val]);/*0.4 1.0 0.0*/
-	r[0]=0.4;
-	g[0]=0.4;
-	b[0]=1;
-	}
-     else if ( val==1)
-        {
-	sprintf ( C->html_color, "#00FF00");/*0.4 1.0 0.0*/
-	r[0]=0.6;
-	g[0]=1;
-	b[0]=0;
-	}
-     else if (val==2)
-        {
-	sprintf ( C->html_color, "#66FF00");/*0.6 1.0 0.0*/
-	r[0]=0.8;
-	g[0]=1;
-	b[0]=0;
-	}
-     else if ( val==3)
-        {
-	sprintf ( C->html_color, "#CCFF00");/*0.8 1.0 0.0*/
-	r[0]=1.0;
-	g[0]=1.0;
-	b[0]=0;
-	}
-     else if (val==4)
-        {
-	sprintf ( C->html_color, "#FFFF00");/*1 1.0 0.0*/    
-	r[0]=1;
-	g[0]=0.85;
-	b[0]=0;
-	}
-     else if ( val==5)
-        {
-	sprintf ( C->html_color, "#FFCC00");/*1.0 0.85 0.0*/  	
-	r[0]=1;
-	g[0]=0.7;
-	b[0]=0;
-	}
-     else if (val==6)
-        {
-	sprintf ( C->html_color, "#FF9900");/*1.0 0.7 0.0*/  
-	r[0]=1;
-	g[0]=0.6;
-	b[0]=0;
-	}
-     else if ( val==7)
-        {
-	sprintf ( C->html_color, "#FF6600");/*1.0 0.6 0.0*/  
-	r[0]=1;
-	g[0]=0.4;
-	b[0]=0;
-	}
-     else if (val==8)
-        {
-	sprintf ( C->html_color, "#FF3300 ");/*1.0 0.4 0.0*/  	
-	r[0]=1;
-	g[0]=0.2;
-	b[0]=0;
-	}   
-     else if (val==9 && val<'a')
-        {
-	sprintf ( C->html_color, "#FF2000");/*1.0 0.2 0.0*/  	
-	r[0]=1.;
-	g[0]=0;
-	b[0]=0;
-	}
-     else if (val==DEFAULT_COLOR || val==NO_COLOR_RESIDUE || val==NO_COLOR_GAP || (val>'A' && val<'z'))
-        {
-	C->html_color[0]='\0';
-	sprintf ( C->html_color_class, "valuedefault");
-	r[0]=1.;
-	g[0]=1;
-	b[0]=1;
-	
-	}
-      else if (val==GAP_COLOR)
-        {
-	C->html_color[0]='\0';
-	sprintf ( C->html_color_class, "valuegap");
-	r[0]=1.;
-	g[0]=1;
-	b[0]=1; 
-	}
-      else if (val==INK_COLOR )
-        {
-	sprintf ( C->html_color, "000000");
-	sprintf ( C->html_color_class, "valueink");
-	r[0]=0.;
-	g[0]=0;
-	b[0]=0;
-	}
+    
      }
-int output_color_format ( Alignment *B,Alignment *S,char *name, \
+int output_color_format ( Alignment *B,Alignment *Sin,char *name, \
 FILE_format *(*vfopen_format)          ( char *),\
 FILE_format *(*print_format_string)    ( char * ,Color *, Color *, FILE_format*),\
 FILE_format *(*print_format_char)      ( int    ,Color *, Color *, FILE_format*),\
@@ -612,6 +620,7 @@ FILE_format* (*vfclose_format)         ( FILE_format *))
     int a, b, c;
     int max_name_len=15;
     int max_len=0;
+    char *buf2, *buf3;
     
     static char *buf;
     int s;
@@ -620,9 +629,38 @@ FILE_format* (*vfclose_format)         ( FILE_format *))
     Color *ink;
     Color *box_c;
     Color *white;
+    Alignment *S;
     
+    S=copy_aln (B, NULL);
     
+    buf2=vcalloc (Sin->len_aln+1, sizeof (char));
+    buf3=vcalloc (  B->len_aln+1, sizeof (char));
+    for ( a=0; a<B->nseq; a++)
+      {
+	int i,n, r;
+	
+	i=name_is_in_list ( B->name[a], Sin->name, Sin->nseq, -1);
+	if (i==-1)continue;
+	sprintf (buf2, "%s", Sin->seq_al[i]);ungap(buf2);
+	sprintf (buf3, "%s", S->seq_al[a]);ungap(buf3);
+	if ( strlen (buf2) !=strlen(buf3))
+	  {
+	    fprintf ( stderr, "\nERROR: Incompatible cache\n");
+	    fprintf ( stderr, "\n%s\n%s", buf2, buf3); exit (EXIT_FAILURE);
+	  }
+	
+	for (n=0,b=0;b<B->len_aln; b++)
+	  {
+	    r=S->seq_al[a][b];
+	    if (!is_gap(r))
+	      {
+		S->seq_al[a][b]=buf2[n++];
+	      }
+	  }
+      }
 
+    S=aln2number(S);
+    vfree (buf2);
     
     box_c=vcalloc ( 1, sizeof (Color));    
     get_rgb_values_format (DEFAULT_COLOR, (white=vcalloc ( 1, sizeof (Color))));	
@@ -662,7 +700,7 @@ FILE_format* (*vfclose_format)         ( FILE_format *))
 		 fps=print_format_char ( fps->eop,white, ink, fps);		
 		 }
 	  
-	   for (b=0; b<=S->nseq; b++)
+	   for (b=0; b<=B->nseq; b++)
 	     {
 	     sprintf (buf,"%-*.*s ",max_len+2, max_len,S->name[b]);
 	     fps=print_format_string ( buf,white, ink, fps);
@@ -674,7 +712,7 @@ FILE_format* (*vfclose_format)         ( FILE_format *))
 	     		 
 	     for (fps->in_seq=1,c=a;c<a+fps->line && c<B->len_aln;c++)
 		{
-		if (b==S->nseq)
+		if (b==B->nseq)
 		   {
 		   n_residues[b]++;
 		   get_rgb_values_format (DEFAULT_COLOR,box_c);
@@ -709,7 +747,11 @@ FILE_format* (*vfclose_format)         ( FILE_format *))
 	    fps=print_format_string ( "\n\n",white, ink, fps);
 	    }
     fps=print_format_string ( "\n\n\n",white, ink,fps);
+    
+    
     vfclose_format( fps);
+    free_aln (S);
+    vfree (n_residues);
     return 1;
 
     }
@@ -733,7 +775,6 @@ FILE_format* (*vfclose_format)         ( FILE_format *))
     int *n_residues;
    
 
-    
     box_c=vcalloc ( 1, sizeof (Color));    
     get_rgb_values_format (DEFAULT_COLOR, (white=vcalloc ( 1, sizeof (Color))));	
     get_rgb_values_format (INK_COLOR,     (ink  =vcalloc ( 1, sizeof (Color))));
@@ -755,16 +796,21 @@ FILE_format* (*vfclose_format)         ( FILE_format *))
 	    {if ( strlen (B->name[a])>max_len)
 		max_len= strlen ( (B->name[a]));
 	    }
-	}	
-    if ( max_len>max_name_len)max_len=max_name_len;
+	}
+
+    if ( vfopen_format==vfopen_ascii)
+      {
+	fps->line+= max_len;
+      }
+    else if ( max_len>max_name_len)max_len=max_name_len;
     
     
     
-   sprintf (buf, "%s, %s(%s)\n%s\nCPU TIME:%d sec.\n",PROGRAM,VERSION,DATE, AUTHOR,  (B->cpu+get_time())/1000);     
-   fps=print_format_string ( buf,white, ink, fps);
-   sprintf (buf, "SCORE=%d\n*\n", S->score_aln);
-   fps=print_format_string ( buf,white, ink, fps);
-   
+    sprintf (buf, "%s, %s(%s)\n%s\nCPU TIME:%d sec.\n%s",PROGRAM,VERSION,DATE, AUTHOR,  (B->cpu+get_time())/1000, (S->generic_comment)?S->generic_comment:"");     
+    fps=print_format_string ( buf,white, ink, fps);
+    sprintf (buf, "SCORE=%d\n*\n", S->score_aln);
+    fps=print_format_string ( buf,white, ink, fps);
+    
    sprintf ( buf2, " BAD AVG GOOD");
    l=strlen(buf2);
    get_rgb_values_format ( DEFAULT_COLOR, box_c);
@@ -787,6 +833,13 @@ FILE_format* (*vfclose_format)         ( FILE_format *))
        sprintf ( buf, ": %3d\n", S->score_seq[a]);
        fps=print_format_string ( buf,white, ink,fps);
        }
+   //Print the Consensus score
+   get_rgb_values_format (S->score_aln/10, box_c);
+   sprintf ( buf, "%-*.*s ", max_len+2,max_len, S->name[S->nseq]);
+   fps=print_format_string ( buf,box_c, ink,fps);
+   sprintf ( buf, ": %3d\n", S->score_aln);
+   fps=print_format_string ( buf,white, ink,fps);
+   
    fps=print_format_string ( "\n",white, ink,fps);
 
 
@@ -804,7 +857,7 @@ FILE_format* (*vfclose_format)         ( FILE_format *))
 	  
 	   for (b=0; b<=S->nseq; b++)
 	     {
-	     if ( b==S->nseq) fps=print_format_string ( "\n",white, ink, fps);
+	     if ( b==S->nseq && print_format_string !=print_ascii_string) fps=print_format_string ( "\n",white, ink, fps);
 	     sprintf (buf,"%-*.*s ",max_len+2,max_len,S->name[b]);
 	     fps=print_format_string ( buf,white, ink, fps);
 	     if(B->output_res_num)
@@ -825,7 +878,8 @@ FILE_format* (*vfclose_format)         ( FILE_format *))
 		else
 		   {
 		   n_residues[b]+=!is_gap(B->seq_al[b][c]);
-		   s=toupper(B->seq_al[b][c]);		   
+		   //s=toupper(B->seq_al[b][c]);
+		   s=B->seq_al[b][c];
 		   if (!is_gap(s) && S->seq_al[b][c]!=NO_COLOR_RESIDUE )
 			{
 			get_rgb_values_format ( S->seq_al[b][c], box_c);			    			
@@ -872,7 +926,7 @@ int       output_color_pdf    ( Alignment *B,Alignment *S, char *name)
       
 #ifndef PS2PDF 
       fprintf (stderr, "\nPDF FORMAT IS NOT SUPPORTED: INSTALL THE PROGRAM PS2PDF\n");
-      exit (1);
+      myexit (EXIT_FAILURE);
 #else
       tmp_name=vtmpnam(NULL);
       
@@ -894,7 +948,7 @@ int       output_reliability_pdf    ( Alignment *B,Alignment *S, char *name)
 
 #ifndef PS2PDF 
       fprintf (stderr, "\nPDF FORMAT IS NOT SUPPORTED: INSTALL THE PROGRAM PS2PDF\n");
-      exit (1);
+      myexit (EXIT_FAILURE);
 #else
       tmp_name=vtmpnam(NULL);
       
@@ -990,7 +1044,7 @@ FILE_format* vfopen_ps ( char *name)
       fps=vcalloc ( 1, sizeof ( FILE_format));
       fps->font=9;
       fps->max_line_ppage=60;
-      fps->line=60;/*N char per line*/
+      fps->line=get_msa_line_length (0, 0);/*N char per line*/
       fps->x0=15;
       fps->y0=750;
       fps->eop='^';
@@ -1119,7 +1173,7 @@ FILE_format* vfopen_html ( char *name)
       fhtml=vcalloc ( 1, sizeof ( FILE_format));
       fhtml->font=11;
       fhtml->max_line_ppage=100000;
-      fhtml->line=70;/*N char per line*/
+      fhtml->line=get_msa_line_length (0, 0);/*N char per line*/
       fhtml->x0=15;
       fhtml->y0=800;
       fhtml->eop='^';
@@ -1216,11 +1270,10 @@ void get_rgb_values_ascii ( int val, Color *C)
 FILE_format* vfopen_ascii ( char *name)
       {
       FILE_format*fascii;
-
       fascii=vcalloc ( 1, sizeof ( FILE_format));
       fascii->font=11;
       fascii->max_line_ppage=100000;
-      fascii->line=70;/*N char per line*/
+      fascii->line=get_msa_line_length (0,0);/*N char per line*/
       fascii->x0=15;
       fascii->y0=800;
       fascii->eop='^';
@@ -1245,8 +1298,8 @@ FILE_format* vfclose_ascii ( FILE_format *fascii)
 int       output_seq_reliability_ascii     ( Alignment *B,Alignment *S, char *name)
 {
   FILE *fp;
-  int a, b;
-  int max_len;
+  int a;
+  int max_len=0;
   for ( a=0; a< B->nseq; a++)
     {if ( strlen (B->name[a])>max_len)
       max_len= strlen ( (B->name[a]));
@@ -1264,7 +1317,7 @@ int       output_seq_reliability_ascii     ( Alignment *B,Alignment *S, char *na
 /* Centre National de la Recherche Scientifique (CNRS) */
 /*and */
 /*Cedric Notredame */
-/*Tue May 10 12:08:44     2005. */
+/*Fri Oct 26 17:03:04     2007. */
 /*All rights reserved.*/
 /*This file is part of T-COFFEE.*/
 /**/
