@@ -1,3 +1,288 @@
+/*DEBUGGING*/
+/*#include "mshell.h"*/
+/*MEMORY MANAGEMENT*/
+#include <float.h>
+#define MY_EPS 1000*DBL_EPSILON
+
+
+#define GIVE_MEMORY_BACK 0
+#define MEMSET0   1
+#define NO_MEMSET0 0
+/*OUTPUT DEFINITIONS*/
+#define  NO_COLOR_RESIDUE 127
+#define  NO_COLOR_GAP 126
+#define  CLOSE_HTML_SPAN -1
+/*SPECIAL_CODES*/
+#define GAP_CODE 60
+/*TYPE DEFINITIONS*/
+
+/*SWITCHES*/
+
+
+#define USED 1
+#define UNUSED 2
+
+
+#define TEMPLATES 1
+#define NOTEMPLATES 0
+
+#define EXTEND 1
+#define RESIZE 2
+
+
+#define ALL               1
+#define SEGMENTS          2
+#define DIAGONALS         3
+
+#define START_STATE       0
+#define END_STATE         1
+
+#define KEEP_CASE         2 /*Hard set in several places*/
+#define LOWER_CASE        0
+#define UPPER_CASE        1
+#define CHANGE_CASE       3
+#define KEEP_GAP          0
+#define RM_GAP            1
+
+#define KEEP_NAME         1
+
+#define CHECK             0
+#define NO_CHECK          1
+#define FORCE             2
+#define STORE             3
+#define FLUSH             4
+
+#define SET               5
+#define UNSET             6
+#define GET               7
+
+#define ON                8
+#define OFF               9
+#define LOCKED_ON         10
+#define LOCKED_OFF        11
+
+#define YES               12
+#define NO                13
+#define MAYBE             14
+
+#define NEVER             15
+#define ALWAYS            16
+#define SOMETIMES         17
+
+#define UPPER             18
+#define LOWER             19
+#define DELETE            20
+#define SWITCHCASE        21 
+
+#define VECTOR            22
+#define NON_VECTOR        23
+#define NON_PROFILE       24
+#define BOOTSTRAP         25
+#define GOP               0
+#define GCP               1
+#define GEP               2
+
+#define BOTTOM             0
+#define TOP                1
+
+#define FORWARD            -1
+#define BACKWARD            1
+
+#define GO_LEFT            -1
+#define GO_RIGHT            1
+
+#define LOCAL            1
+#define GLOBAL           2
+#define LALIGN           3
+#define MOCCA            4
+
+#define TRUE             1
+#define FALSE            0
+
+#define NEW              1
+#define OLD              0
+
+#define RANDOM           0
+#define DETERMINISTIC    1
+
+#define GREEDY           1
+#define NON_GREEDY       0
+
+#define IS_FATAL         1
+#define IS_NOT_FATAL     0
+
+#define OPTIONAL         1
+#define NON_OPTIONAL     0
+
+#define GV_MAXIMISE      1
+#define GV_MINIMISE      0
+
+#define MAXIMISE      1
+#define MINIMISE      0
+
+#define ALLOWED          0
+#define FORBIDEN         -99999999
+#define END_ARRAY        -99999990
+#define SOFT_COPY 1
+#define HARD_COPY 2
+
+#define VERY_SLOW 0
+#define SLOW 1
+#define FAST 2
+#define VERY_FAST 3
+#define SUPER_FAST 4
+#define ULTRA_FAST 5
+
+#define CODE 1
+#define DECODE 2
+
+/*Identity measure*/
+#define UNGAPED_POSITIONS 1
+#define ALIGNED_POSITIONS 2
+#define AVERAGE_POSITIONS 3
+#define NOMATRIX         NULL
+#define NOGROUP          NULL
+#define NOALN            NULL
+
+/*SIZE DEFINITIONS*/
+#define SIZE_OF_INT      10
+#define UNDEFINED        FORBIDEN
+#define UNDEFINED_INT    UNDEFINED
+#define UNDEFINED_FLOAT  UNDEFINED
+#define UNDEFINED_DOUBLE UNDEFINED
+#define UNDEFINED_CHAR   125
+#define UNDEFINED_SHORT  -125
+#define UNDEFINED_2      0
+#define UNDEFINED_RESIDUE '>'
+
+#define FACTOR           1
+#define MAX_N_SEQ        1
+#define MAX_N_ALN        1
+#define MAX_LEN_ALN      1
+#define MAX_N_LIST       100
+
+#define COMMENT_SIZE     1000
+#define MAXNAMES         100
+#define FILENAMELEN 	 500            /* Max. file name length */
+#define MAX_N_PARAM      2000
+#define MAX_PARAM_LEN    200
+#define MAX_LINE_LENGTH  10000
+#define ALN_LINE_LENGTH  60
+#define SHORT_STRING     10
+#define STRING           300
+#define LONG_STRING      1000
+#define VERY_LONG_STRING 10000
+
+#define AA_ALPHABET            "acdefghiklmnpqrstvwy-ACDEFGHIKLMNPQRSTVWY"
+#define DNA_ALPHABET           "AGCTUNRYMKSWHBVD-agctunrymkswhbvd"
+#define BLAST_AA_ALPHABET      "arndcqeghilkmfpstwyvbzx*"
+#define NAMES_ALPHABET         "1234567890ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz_|-!%@&#-+=."
+
+#define SIZEOF_AA_MAT   60
+#define GAP_LIST         "-.#*~"
+#define SSPACE           "   "
+#define SCORE_K          10
+#define PAVIE_MAT_FACTOR 1000
+#define MAXID            100
+#define CLEAN_FUNCTION   NULL
+
+#define TRACE_TYPE       int
+#define MAX_LEN_FOR_DP   600
+#define MATCH            1
+#define UNALIGNED        2
+#define GAP              3
+
+#define MNE 3
+
+/*CODE SHORT CUTS*/
+
+/*1-COMMAND LINE PROCESSING*/
+#define GET_COMMAND_LINE_INFO ((strncmp ( argv[1], "-h",2)==0)||(strncmp ( argv[1], "-man",4)==0)||(strncmp ( argv[1], "-",1)!=0))
+#define NEXT_ARG_IS_FLAG ((argc<=(a+1)) ||(( argv[a+1][0]=='-') && !(is_number(argv[a+1]))))
+
+
+/*UTIL MACROS*/
+#define GET_CASE(f,c) ((f==UPPER_CASE)?toupper(c):((f==LOWER_CASE)?tolower(c):c))
+
+#define SWAP(x,y) {x=x+y;y=x+y; x=y-x; y=y-2*x;}
+#define SWAPP(x,y,tp) {tp=y;y=x;x=tp;}
+
+#define MAX(x, y) (((x) >(y)) ? (x):(y))
+#define MAX2(x, y) (((x) >(y)) ? (x):(y))
+#define MAX3(x,y,z) (MAX(MAX(x,y),z))
+#define MAX4(a,b,c,d) (MAX(MAX(a,b),MAX(c,d)))
+#define MAX5(a,b,c,d,e) (MAX2((MAX3(a,b,c)),(MAX2(d,e))))
+#define MAX6(a,b,c,d,e,f) (MAX2((MAX3(a,b,c)),(MAX3(c,d,e))))
+
+#define MIN(x, y) (((x) <(y)) ? (x):(y))
+#define FABS(x) ((x<0)?(-x):(x))
+#define is_defined(x) ((x==UNDEFINED)?0:1)
+#define a_better_than_b(x,y,m) ((m==1)?(((x)>(y))?1:0):(((x)<(y))?1:0))
+#define is_in_range(x,min,max) ((x>=min && x<=max)?1:0)
+/*#define bod_a_b(x,y,m)   ((m==1)?(MAX((x),(y))):(MIN((x),(y))))
+#define bo_a_b(x,y,m)    ((x==UNEFINED)?y:((y==UNDEFINED)?x:bod_a_b(y,y,m)))
+#define best_of_a_b(x,y,m)   ((x==UNDEFINED && y==UNDEFINED)?(UNDEFINED):(bo_a_b(x,y,m)))
+*/
+
+
+#define DIE(x)  HERE(x);exit(0);
+#define best_of_a_b(x,y,m) ((m==1)?(MAX((x),(y))):(MIN((x),(y))))
+
+#define strm(x,y)            ((vstrcmp((x),(y))==0)?1:0)
+#define strnm(x,y,n)           ((vstrncmp((x),(y),(n))==0)?1:0)
+#define strm2(a,b,c)         (strm(a,b) || strm(a,c))
+#define strm3(a,b,c,d)       (strm2(a,b,c) || strm(a,d))
+#define strm4(a,b,c,d,e)     (strm2(a,b,c) || strm2(a,d,e))
+#define strm5(a,b,c,d,e,f)   (strm2(a,b,c) || strm3(a,d,e,f))
+#define strm6(a,b,c,d,e,f,g) (strm3(a,b,c,d) || strm3(a,e,f,g))
+#define declare_name(x) (x=vcalloc (MAX(FILENAMELEN,L_tmpnam)+1, sizeof (char))) 
+#define is_parameter(x) (x[0]=='-' && !isdigit(x[1])) 
+
+/*Freing functions*/
+#define free_2(a, b)            free(a);free(b)
+#define free_1(a)               free(a)
+#define free_3(a, b, c)         free_2(a,b);free_1(c)
+#define free_4(a, b, c,d)       free_2(a,b);free_2(c,d)
+#define free_5(a, b, c,d,e)     free_3(a,b,e);free_2(c,d)
+#define free_6(a, b, c,d,e,f)   free_3(a,b,e);free_3(c,d,f)
+#define free_7(a, b, c,d,e,f,g) free_3(a,b,e);free_4(c,d,f,g)
+/*2-FILE PARSING*/
+#define SEPARATORS "\n \t,;"
+#define LINE_SEPARATOR "\n#TC_LINE_SEPARATOR\n"
+#define TC_REC_SEPARATOR "#### TC REC SEPARATOR ###"
+
+/*END 1-*/
+
+
+/*WIDOWS/UNIX DISTINCTIONS
+#if defined(_WIN32) || defined(__WIN32__) ||  defined(__WINDOWS__) || defined(__MSDOS__) || defined(__DOS__) || defined(__NT__) || defined(__WIN32__)
+#define WIN32
+#define TO_NULL_DEVICE " >nul"
+#define    NULL_DEVICE "nul"
+#define CWF "/" 
+#else
+#define TO_NULL_DEVICE " >/dev/null 2>&1"
+#define    NULL_DEVICE "/dev/null"
+*/
+
+#if defined(_WIN32) || defined(__WIN32__) ||  defined(__WINDOWS__) || defined(__MSDOS__) || defined(__DOS__) || defined(__NT__) || defined(__WIN32__)
+#define WIN32
+#define TO_NULL_DEVICE " >>t_coffee.log"
+#define    NULL_DEVICE "t_coffee.log"
+#define CWF "/" /*ClustalW Flag*/
+#else
+#define TO_NULL_DEVICE " >>/dev/null 2>&1"
+#define    NULL_DEVICE "/dev/null"
+
+
+#define CWF "-" /*ClustaW Flag*/
+#endif
+
+/*Generic Data*/
+#define MAIL "cedric.notredame@europe.com"
+#define URL "http://www.tcoffee.org"
+
+#define PERL_HEADER "#!/usr/bin/env perl"
 typedef struct
     {
     char *mode;
@@ -40,7 +325,7 @@ struct Sequence
       int contains_gap;   /*set to 1 if gaps are to be kept*/
       char *type;         /*PROTEIN, DNA*/
       Weights *W;         /*Associated weights*/
-      char template_file[100];
+      char template_file[FILENAMELEN+1];
       struct Template **T;
       
 };
@@ -59,10 +344,10 @@ typedef struct Template Template;
 
 struct X_template 
 {
-  char seq_name[100];
-  char template_type[100];
-  char template_name[100];
-  char template_file[100];
+  char seq_name[FILENAMELEN+1];
+  char template_type[FILENAMELEN+1];
+  char template_name[FILENAMELEN+1];
+  char template_file[FILENAMELEN+1];
   struct P_template *VP;
   struct S_template *VS;
   struct R_template *VR;
@@ -75,7 +360,6 @@ typedef struct X_template X_template;
 struct P_template
 {
   char pdb_id[100];
-  char public_pdb;
 };
 typedef struct P_template P_template;
 
@@ -119,17 +403,19 @@ struct Alignment
     int max_len;
     int min_len;   
     int *  len;
+      //int *weight;  
     int declared_len;
     int max_n_seq;
     int nseq;
     int len_aln;
-
+/*Generic Information*/
+      char *generic_comment;
 /*Sequence Information*/
     char **file;
     char **seq_comment;
     char **aln_comment;
     char **name;
-
+      
     char **expanded_order;
     char **tree_order;
     char **seq_al;
@@ -165,7 +451,7 @@ struct Alignment
 /*Input/Output Options*/
     int output_res_num;
     int residue_case; /*1 for lower, 0 for Upper, 2 for keeping unchanged*/
-      
+      int expand;
 /*Must Not be copied*/
      int used;
      int num;     
@@ -174,6 +460,7 @@ struct Alignment
     struct Alignment * A;  
       /*Misc*/
     int random_tag;
+     
     };
     
 typedef struct Alignment Alignment;
@@ -213,8 +500,13 @@ Sequence * fill_sequence_struc ( int nseq, char **sequences, char **seq_name);
 Sequence * cw_read_sequences ( char *seq_name);
 Sequence * get_sequence_type (Sequence *S);
 Alignment* get_aln_type (Alignment *A);
+
 char     * get_string_type   (char *string);
 
+char *store_seq_type (char *val);
+char *retrieve_seq_type ();
+char *unset_seq_type ();
+char *set_seq_type (int mode, char *val);
 
 void get_sequence (char *seq_file,int *NSEQ, char ***SEQ, char ***SN, int **sl, int *min, int *max);
 
@@ -372,43 +664,69 @@ typedef struct node {		/* phylogenetic tree structure */
         char name[64];
 } stree, *treeptr;
 
+void		*ckalloc(size_t);
+void 		* ckvrealloc(void *,size_t);
+void 		ckfree(void *);
+
+int readseqs(char *saga_file,char ***SAGA_SEQ, char*** SAGA_NAMES, int ***SAGA_LEN) ;/*first_seq is the #no. of the first seq. to read */
+typedef struct treesim{
+  float w;
+  float uw;
+  float d;
+  
+  float max_w;
+  float max_uw;
+  float max_d;
+  
+    }Tree_sim;
+
 typedef struct tnode *NT_node;
 
 typedef struct tnode{
-    int visited;
-    char *name;
-    NT_node parent;
-    NT_node left;
-    NT_node right;
-    NT_node bot;
-    int isseq;
-    int seq;
-    int maxnseq;
-    int nseq;
-    int *lseq;
-    int *lseq2;
-    float dist;
-    float bootstrap;
-    float dp;
-    int order;
+  int visited;
+  char *name;
+  char *file;
+  NT_node parent;
+  NT_node left;
+  NT_node right;
+  NT_node bot;
+  int isseq;
+  int seq;
+  int maxnseq;
+  int nseq;
+  int *lseq;
+  int *lseq2;
+  int *idist;//contains distances to the root, in nodes
+  int *ldist;//contains real distances *1000;
+  float dist;
+  float bootstrap;
+  float dp;
+  int order;
+  int aligned;
   int leaf;/*Number of leave below the considered node*/
   int node;/*Number of nodes below the considered node*/
-    int group;
-    float score;
+  int group;
+  float score;
+  int align;
+  char *seqal;
+  int index;
     }Treenode;
 NT_node main_prune_tree ( NT_node T, Sequence *S);
-int prune_tree ( NT_node T, Sequence *S);
+NT_node prune_tree ( NT_node T, Sequence *S);
 /*********************************************************************/
 /*                                                                   */
 /*                                   dpa_tree_manipulation           */
 /*                                                                   */
 /*                                                                   */
 /*********************************************************************/
-char *tree2Ngroup (Alignment *A, NT_node T, int max_n, char *fname);
+char *tree2Ngroup (Alignment *A, NT_node T, int max_n, char *fname, char *mat4dist);
 int tree2group_file ( NT_node T,Sequence *S, int maxnseq, int minsim, char *name);
 
-NT_node tree2dpa_tree (NT_node T, Alignment *A);
+NT_node seq2dpa_tree  (Sequence *S, char *align_mode);
+NT_node tree2dpa_tree (NT_node T, Alignment *A, char *matrix4distance);
 FILE * tree2group ( NT_node T,Sequence *S,int maxnseq, int mindist,char *name, FILE *fp);
+
+
 NT_node  tree2collapsed_tree (NT_node T, int n, char **string);
 
 /*********************************************************************/
@@ -417,6 +735,10 @@ NT_node  tree2collapsed_tree (NT_node T, int n, char **string);
 /*                                                                   */
 /*                                                                   */
 /*********************************************************************/
+int main_compare_cog_tree (NT_node T1, char *cogfile);
+int main_compare_aln_tree (NT_node T1, Alignment *A, FILE *fp);
+int compare_aln_tree (NT_node T, Alignment *A, int *n, FILE *fp);
+
 NT_node main_compare_trees ( NT_node T1, NT_node T2, FILE *fp);
 float compare_trees ( NT_node T1, NT_node T2, int nseq, int mode);
 float search_node ( NT_node B, NT_node T, int nseq, int mode);
@@ -433,9 +755,10 @@ int compare_branch2 ( int *b1, int *b2, int n);
 /*                                                                   */
 /*                                                                   */
 /*********************************************************************/
+NT_node similarities_file2tree (char *mat);
 NT_node tree_compute ( Alignment *A, int n, char ** arg_list);
 NT_node aln2fj_tree(NT_node T, Alignment *A, int limit,char* mode);
-Alignment * filter_aln4tree (Alignment *A, int n,char* mode);
+Alignment * filter_aln4tree (Alignment *A, int n,int fg,char* mode);
 
 /*********************************************************************/
 /*                                                                   */
@@ -462,7 +785,9 @@ int tree2nnode ( NT_node R);
 
 FILE* display_tree ( NT_node R, int n, FILE *fp);
 void clear_tree (NT_node T);
-void display_leaf ( NT_node T);
+int display_leaf ( NT_node T, FILE *fp);
+int display_leaf_below_node ( NT_node T, FILE *fp);
+NT_node display_leaf_nb (NT_node T, int n, FILE *fp, char *name);
 
 NT_node reroot_tree ( NT_node TREE, NT_node T);
 NT_node straighten_tree ( NT_node P, NT_node C, float new_dist);
@@ -479,6 +804,12 @@ int ** make_sub_tree_list ( NT_node **T, int nseq, int n_node);
 void make_all_sub_tree_list ( NT_node N, int **list, int *n);
 void make_one_sub_tree_list ( NT_node T, int *list);
 NT_node main_read_tree(char *treefile);
+
+NT_node new_read_tree ( char *teefile);
+NT_node new_get_node (NT_node T, FILE *fp);
+
+
+NT_node** simple_read_tree(char *treefile);
 NT_node** read_tree(char *treefile, int *nnodes,int nseq, char **seq_names);
 FILE * create_tree(NT_node ptree, NT_node parent,int *numseq,int  *ntotal,int  *nnodes,NT_node **lu, FILE *fp);
 NT_node declare_tree_node (int nseq);
@@ -490,6 +821,29 @@ float calc_mean(NT_node nptr, float *maxdist, int nseq,NT_node **lu);
 NT_node insert_root(NT_node p, float diff);
 float calc_root_mean(NT_node root, float *maxdist, int neq, NT_node **lu);
 NT_node reroot(NT_node ptree, int nseq, int ntotal, int nnodes, NT_node **lu);
+
+
+Alignment *seq2seq_chain (Alignment *A,Alignment *B, char *arg);
+
+float display_avg_bootstrap ( NT_node T);
+float tree2tot_dist ( NT_node T, int mode);
+int tree2n_branches(NT_node T, int mode);
+int **display_tree_from_node (NT_node T, int up, int down, int **array);
+NT_node tree2node ( char *name, NT_node T);
+int new_tree_sort ( char *name, NT_node T);
+
+
+
+NT_node * read_tree_list (Sequence *S);
+int treelist2groups (Sequence *S, Sequence *ST);
+int treelist2splits (Sequence *S, Sequence *ST);
+int treelist2leafgroup ( Sequence *S, Sequence *TS);
+int ***tree2dist ( NT_node T, Sequence *S, int ***d);
+int treelist2lti ( Sequence *S, Sequence *TS, char *nb);
+
+float simple_tree_cmp (NT_node T1, NT_node T2,Sequence *S, int mode);
+
+int treelist2dmat ( Sequence *S);
 /* General purpose header file - rf 12/90 */
 
 #ifndef _H_general
@@ -514,7 +868,7 @@ typedef int Boolean;			/* Is already defined in THINK_C */
 /* Centre National de la Recherche Scientifique (CNRS) */
 /*and */
 /*Cedric Notredame */
-/*Tue May 10 12:08:44     2005. */
+/*Fri Oct 26 17:03:04     2007. */
 /*All rights reserved.*/
 /*This file is part of T-COFFEE.*/
 /**/
