@@ -31,7 +31,7 @@ static void     get_msf_seq(char *,char *,int *,char *,int);
 static void check_infile(int *);
 
 
-static int res_index(char *,char);
+
 
 static int count_clustal_seqs(void);
 static int count_msf_seqs(void);
@@ -51,11 +51,11 @@ static char chartab[128];
 void fill_chartab(void)	/* Create translation and check table */
 {
 	register int i;
-	register char c;
+	register int c;
 
 
 	for(i=0;i<128;chartab[i++]=0);
-	for(i=0;c=amino_acid_codes[i];i++)
+	for(i=0,c=0;c<=amino_acid_codes[i];i++)
 		chartab[c]=chartab[tolower(c)]=c;
 
 }
@@ -146,8 +146,8 @@ static void get_clustal_seq(char *sname,char *seq,int *len,char *tit,int seqno)
 static void get_seq(char *sname,char *seq,int *len,char *tit)
 {
 	static char *line;
-	int i, offset;
-        unsigned char c;
+	int i, offset, c=0;
+
 	if ( !line)line=vcalloc ( (MAXLINE+1), sizeof (char));
 	switch(seqFormat) {
 
@@ -216,7 +216,7 @@ static void get_seq(char *sname,char *seq,int *len,char *tit)
 			}
 		break;
 /***********************************************/
-		case PEARSON:
+	case (PEARSON):
 			
 			while(*line != '>')
 				{
@@ -531,20 +531,11 @@ static int count_msf_seqs(void)
 
 
 
-static int res_index(char *t,char c)
-{
-	register int i;
-	
-	for(i=0;t[i] && t[i] != c;i++)
-		;
-	if(t[i]) return(i);
-	else return -1;
-}
 /*********************************COPYRIGHT NOTICE**********************************/
 /* Centre National de la Recherche Scientifique (CNRS) */
 /*and */
 /*Cedric Notredame */
-/*Tue May 10 12:08:44     2005. */
+/*Fri Oct 26 17:03:04     2007. */
 /*All rights reserved.*/
 /*This file is part of T-COFFEE.*/
 /**/
