@@ -4,7 +4,7 @@
 #include <stdarg.h>
 #include <string.h>
 #include <stddef.h>
-
+#include <ctype.h>
 #include "io_lib_header.h"
 #include "util_lib_header.h"
 #include "dp_lib_header.h" 
@@ -14,7 +14,72 @@
 
 #define FATAL "fatal:reformat_struc"
 
+char * process_repeat (char *aln, char *seq, char *pdb)
+{
+  char *tf, *file, *name;
+  Alignment *A, *A2;
+  Sequence *S, *P;
+  int r1, r2, is_r1, is_r2, l1=0, l2=0, a;
+  int *pos;
+  FILE *fp;
+  
+  A=main_read_aln (aln, NULL);
+  for (a=0; a<A->nseq; a++)ungap(A->seq_al[a]);
+  
+  S=main_read_seq (seq);
+  P=get_pdb_sequence (pdb);
+  
 
+  A2=align_two_sequences (S->seq[0], P->seq[0], "pam250mt", -10, -1, "myers_miller_pair_wise");
+  
+  pos=vcalloc ( A2->len_aln+1, sizeof (int));
+
+  for (l1=0, l2=0,a=0; a< A2->len_aln; a++)
+    {
+      
+      r1=A2->seq_al[0][a];
+      r2=A2->seq_al[1][a];
+      
+      is_r1=1-is_gap(r1);
+      is_r2=1-is_gap(r2);
+      
+      l1+=is_r1;
+      l2+=is_r2;
+      
+      if (!is_r1);
+      else
+	{
+	  pos[l1]=l2;
+	}
+    }
+  tf=vtmpnam(NULL);
+  fp=vfopen (tf, "w");
+  for (a=0; a<A->nseq; a++)
+    {
+      int *coor, b, c;
+      
+      name=A->name[a];
+      file=vtmpnam (NULL);
+      coor=string2num_list2 (name, "-");
+
+      //Check the compatibility between the guide sequence and the coordinates
+      for ( c=0,b=coor[1]-1; b<coor[2]; b++, c++)
+	{
+
+	  if (tolower(A->seq_al[a][c])!=tolower(S->seq[0][b]))
+	    printf_exit (EXIT_FAILURE, stderr, "Incompatibility between the repeat [%s] and the master Sequence [%s]\n%s",A->name[a], seq, A->seq_al[a]);
+	}
+   
+      printf_system ( "extract_from_pdb %s -coor %d %d -seq_field SEQRES> %s", pdb,pos[coor[1]-1],pos[coor[2]-1], file);
+      fprintf (fp, ">%s _P_ %s\n", name, file);
+      vfree (coor);
+    }
+  vfclose (fp);
+  return tf;
+}
+  
+  
+  
 char *     normalize_pdb_file  (char *name_in, char *seq, char *out_file)
 {
   char command[1000];
@@ -33,7 +98,8 @@ char *     normalize_pdb_file  (char *name_in, char *seq, char *out_file)
  
  if ( !is_pdb_file(name))
     {
-      fprintf(stderr, "\nERROR[normalize_pdb_file]: %s is not a pdb file[FATAL:%s]\n", name, PROGRAM);
+      fprintf(stdout, "\nERROR[normalize_pdb_file]: %s is not a pdb file[FATAL:%s]\n", name, PROGRAM);
+      myexit (EXIT_FAILURE);
     }
 
   S=get_pdb_sequence (name);
@@ -186,10 +252,12 @@ Ca_trace * read_ca_trace (char *name, char *seq_field )
   char *tp_name=NULL;
   char command[10000];
   
+
   if ( !is_simple_pdb_file (name))
     {
       tp_name=vtmpnam (NULL);
       sprintf ( command, "extract_from_pdb -seq_field %s  -infile %s -atom ALL -chain FIRST -mode simple> %s",seq_field, check_file_exists(name), tp_name);
+      if ( getenv4debug ("DEBUG_EXTRACT_FROM_PDB"))fprintf ( stderr, "\n[DEBUG_EXTRACT_FROM_PDB:read_ca_trace] %s\n", command);
       my_system ( command);
     }
   else
@@ -210,7 +278,7 @@ Ca_trace * simple_read_ca_trace (char *tp_name )
 	char res;
 	char *buf;
 	Ca_trace *T=NULL;
-	int res_num, last_res_num;
+	int res_num=0, last_res_num=0;
 
 	
 	buf=vcalloc ( VERY_LONG_STRING, sizeof (char));
@@ -782,7 +850,46 @@ char *string2contacts (char *seq,char *name, char *comment, float T)
       return result;
 }
 
-short **struc2nb (char *name,char *seq, char *comment, float Threshold, char *atom)
+
+char **struclist2nb (char *name,char *seq, char *comment, float Threshold, char *atom, char *output)
+{
+  char *list, **pdb;
+  int a;
+  char **R, *tmpf;
+  
+  tmpf=vtmpnam (NULL);
+  
+  list=strstr ( comment, "_P_")+3;
+  if ( !strstr (comment, "_P_"))return NULL;
+  else
+    {
+      pdb=string2list ( strstr ( comment, "_P_")+3);
+    }
+  
+  for (a=1; a<atoi(pdb[0]); a++)
+    {
+      char com[1000];
+      sprintf ( com, "_P_ %s", pdb[a]);
+      R=struc2nb (name, seq, com, Threshold, atom, tmpf);
+    }
+  if (output==NULL|| strstr (output, "fasta"))
+    {
+      for (a=0; a< strlen (seq); a++)
+	if (isupper (R[a][a]))fprintf (stdout, ">%s R=%d T=%.2f %s\n%s\n", name, a+1, Threshold, comment, R[a]);
+    }
+  else
+    {
+      FILE *fp;
+      char c;
+      
+      fp=vfopen (tmpf, "r");
+      while ( (c=fgetc(fp))!=EOF)fprintf (stdout, "%c", c);
+      vfclose (fp);
+    }
+  return NULL;
+}
+
+char **struc2nb (char *name,char *seq, char *comment, float Threshold, char *atom, char *output)
 {
   char *struc_file;
   char *struc_name;
@@ -790,30 +897,138 @@ short **struc2nb (char *name,char *seq, char *comment, float Threshold, char *at
   Atom *A1, *A2;
   int a, b;
   short **hasch;
+  FILE *fp;
   float d;
   char command[10000];
+  static char **R;
   
   struc_file=vtmpnam (NULL);
-  struc_name=strstr(name, "_S_");
+  declare_name (struc_name);
+  
+  sscanf ( strstr(comment, "_P_"), "_P_ %s", struc_name);
+  //struc_name=strstr(name, "_S_");
+  
+  if (!R)
+    {
+      int l;
+      l=strlen (seq);
+      R=declare_char (l+1, l+1);
+      for ( a=0; a<l; a++)
+	{
+	  sprintf ( R[a], "%s", seq);
+	  lower_string (R[a]);
+	}
+    }
   if ( atom) atom=translate_string (atom, "_", " ");
   if ( !struc_name) return NULL;
   
   else 
     {
-      struc_name+=3;
+      //struc_name+=3;
       if ( check_file_exists (struc_name) && is_simple_pdb_file(struc_name))
 	{
 	  sprintf (command, "cp %s %s", name, struc_file);
 	}
       else
 	{
-	  sprintf ( command, "extract_from_pdb -infile %s -atom %s -mode simple -force >%s",name,(atom==NULL)?"ALL":atom, struc_file);
+	  sprintf ( command, "extract_from_pdb -infile %s -atom %s -mode simple -force >%s",struc_name,(atom==NULL)?"ALL":atom, struc_file);
 	}
+  
       my_system (command);
     }
   T=read_ca_trace (struc_file, "ATOM");
   hasch=declare_short (T->len, T->len);
+  
+  if (!R)
+    {
+      int l;
+      l=strlen (seq);
+      R=declare_char (l+1, l+1);
+      for ( a=0; a<l; a++)
+	{
+	  sprintf ( R[a], "%s", seq);
+	  lower_string (R[a]);
+	}
+    }
+  for ( a=0; a< T->n_atom; a++)
+    for ( b=0; b< T->n_atom; b++)
+      {
+	A1=T->structure[a];A2=T->structure[b];
+	d=get_atomic_distance (A1, A2);
 
+	if ( d<Threshold)
+	  {
+	    hasch[A1->res_num-1][A2->res_num-1]=1;
+	  }
+      }
+  fp=vfopen (output, "a");
+  fprintf ( fp, "#Distance_map_format_01\n#Sequence %s with T= %.2f", struc_name, Threshold);
+  for ( a=0; a<T->len; a++)
+    {
+      int c;
+      c=change_residue_coordinate ( T->seq,seq,a);
+      
+      if ( c!=-1 && seq)
+	{
+	  char r1, r2;
+	  r1=(T->seq)[a];
+	  r2=seq[c];
+	  r1=tolower(r1);r2=tolower(r2);
+	  if ( r1!=r2) continue;
+	  R[c][c]=toupper (R[c][c]);
+	  fprintf (fp, "\n%s Residue %d ",struc_name,c+1);
+	  for ( b=0; b<T->len; b++)
+	    {
+	      int d;
+	      char r3, r4;
+	      r3=(T->seq)[a];r4=seq[c];
+	      r3=tolower(r3);r4=tolower(r4);
+	      if ( r3!=r4) continue;
+	      
+	      d=change_residue_coordinate (T->seq,seq,b);
+	      
+	      if ( hasch[a][b] && d!=-1)
+		{
+		  fprintf (fp, "%d ",d+1);
+		  R[c][d]=toupper (R[c][d]);
+		}
+	    }
+	  fprintf (fp, ";");
+	}
+    }
+  free_short (hasch, -1);
+  fprintf (fp, "\n");
+  return R;
+}
+
+short **seq2nb (char *seq, char *pdb, float Threshold, char *atom)
+{
+  char *struc_file;
+  char *struc_name;
+  Ca_trace *T;
+  Atom *A1, *A2;
+  int a, b;
+  short **hasch;
+  short **result; //Contains the result for every residue of seq
+  float d;
+  char command[10000];
+
+  // get a clean pdb file
+  struc_file=vtmpnam (NULL);
+  if ( check_file_exists (struc_file) && is_simple_pdb_file(struc_name))
+    {
+      sprintf (command, "cp %s %s", pdb, struc_file);
+    }
+  else
+    {
+      sprintf ( command, "extract_from_pdb -infile %s -atom %s -mode simple -force >%s",pdb,(atom==NULL)?"ALL":atom, struc_file);
+    }
+  my_system (command);
+  
+  //Read and hasch the PDB file
+  T=read_ca_trace (struc_file, "ATOM");
+  hasch=declare_short (T->len, T->len);
+  result=declare_short (strlen (seq)+1, strlen (seq)+1);
   for ( a=0; a< T->n_atom; a++)
     for ( b=0; b< T->n_atom; b++)
       {
@@ -826,32 +1041,32 @@ short **struc2nb (char *name,char *seq, char *comment, float Threshold, char *at
 	  }
       }
   
-  fprintf ( stdout, "#Distance_map_format_01\n#Sequence %s with T= %.2f", struc_name, Threshold);
   for ( a=0; a<T->len; a++)
     {
       int c;
       c=change_residue_coordinate ( T->seq,seq,a);
       if ( c!=-1)
 	{
-	  fprintf ( stdout, "\n%s Residue %d ",struc_name,c+1);
 	  for ( b=0; b<T->len; b++)
 	    {
 	      int d;
 	      d=change_residue_coordinate (T->seq,seq,b);
+	      
 	      if ( hasch[a][b] && d!=-1)
-		fprintf (stdout, "%d ",d+1);
+		{
+		  result[c][result[c][0]++]=d;
+		}
 	    }
-	  fprintf (stdout, ";");
 	}
     }
-  fprintf (stdout, "\n");
-  return hasch;
+  free_short (hasch, -1);
+  return result;
 }
 /*********************************COPYRIGHT NOTICE**********************************/
 /* Centre National de la Recherche Scientifique (CNRS) */
 /*and */
 /*Cedric Notredame */
-/*Tue May 10 12:08:44     2005. */
+/*Fri Oct 26 17:03:04     2007. */
 /*All rights reserved.*/
 /*This file is part of T-COFFEE.*/
 /**/
