@@ -7,9 +7,7 @@
 #include <assert.h>
 #include <math.h>
 #include <stdarg.h>
-#include <sys/times.h>
-#include <signal.h>
-#include <unistd.h>
+
 
 #include "io_lib_header.h"
 #include "util_lib_header.h"
@@ -21,6 +19,11 @@
 /*                                                                      */
 /*                                                                      */
 /************************************************************************/ 
+Constraint_list * declare_constraint_list_simple ( Sequence *S)
+{
+  return declare_constraint_list (S, NULL, NULL, 0, NULL, NULL);
+}
+
 Constraint_list * declare_constraint_list ( Sequence *S, char *name, int **L, int ne,FILE *fp, int **M)
     {
     Constraint_list *CL;
@@ -53,77 +56,7 @@ Constraint_list * declare_constraint_list ( Sequence *S, char *name, int **L, in
     
     return CL;
     }
-/*
-Constraint_list *cache_dp_value4constraint_list ( char mode[],Constraint_list *CL)
-    {
-	static char dp_mode[100];
-	static int gop;
-	static int f_gop;
-	static int f_gep;
-	static int gep;
-	static int TG_MODE;
-	static int F_TG_MODE;
-	static int maximise;
-	static char matrix_for_aa_group[100];
-	static int diagonal_threshold;
-	static int ktup;
-	static int fasta_step;
-	static int use_fragments;
-	static int extend_jit;
-	static int (*get_dp_cost)(Alignment*, int **, int, int*, int, int **, int, int*, int, struct Constraint_list *);
-        static int (*evaluate_residue_pair)(struct Constraint_list *, int, int, int, int);
 
-
-	if ( strm (mode, "cache"))
-	   {
-	   sprintf (dp_mode, "%s", CL->dp_mode);
-	   gop=CL->gop;
-	   f_gop=CL->f_gop;
-	   gep=CL->gep;
-	   f_gep=CL->f_gep;
-	   TG_MODE=CL->TG_MODE;
-	   F_TG_MODE=CL->F_TG_MODE;
-	   maximise=CL->maximise;
-	   sprintf (matrix_for_aa_group, "%s", CL->matrix_for_aa_group);
-	   diagonal_threshold=CL->diagonal_threshold;
-	   ktup=CL->ktup;
-	   fasta_step=CL->fasta_step;
-	   use_fragments=CL->use_fragments;
-	   extend_jit=CL->extend_jit;
-	   get_dp_cost=CL->get_dp_cost;
-	   evaluate_residue_pair=CL->evaluate_residue_pair;
-	   return CL;
-	   }
-	else if ( strm (mode, "restaure"))
-	   {
-	   sprintf ( CL->dp_mode, "%s",dp_mode);
-	   CL->gop=gop;
-	   CL->f_gop=f_gop;
-	   CL->gep=gep;
-	   CL->f_gep=f_gep;
-	   
-	   CL->TG_MODE=TG_MODE;
-	   CL->F_TG_MODE=F_TG_MODE;
-	   CL->maximise=maximise;
-	   sprintf ( CL->matrix_for_aa_group, "%s", matrix_for_aa_group);
-	   CL->diagonal_threshold=diagonal_threshold;
-	   CL->fasta_step=fasta_step;
-	   CL->ktup=ktup;
-	   CL->use_fragments=use_fragments;
-	   CL->extend_jit=extend_jit;
-	   CL->get_dp_cost=get_dp_cost;
-	   CL->evaluate_residue_pair=evaluate_residue_pair;
-	   return CL;
-	   
-	   }
-	else
-	   {
-	       fprintf (stderr, "\nUNKNOWN MODE %s for cache_dp_value4comstraint_list[FATAL]", mode);
-	       crash ( "");
-	       return 0;
-	   }
-    }
-*/	    
 Constraint_list *free_constraint_list4lib_computation (Constraint_list *CL)
 {
   
@@ -138,12 +71,15 @@ Constraint_list *duplicate_constraint_list4lib_computation (Constraint_list *CL)
   Constraint_list *SCL;
   SCL=vcalloc (1, sizeof ( Constraint_list));
   SCL[0]=CL[0];
+  SCL->S=CL->S;
+  SCL->RunName=CL->RunName;
   SCL->L=NULL;
   SCL->max_L_len=0;
   SCL->M=NULL;
   SCL->ne=0;
   SCL->residue_indexed=0;
   SCL->residue_index=NULL;
+  
   return SCL;
 }
 Constraint_list *duplicate_constraint_list_soft (Constraint_list *CL)
@@ -181,16 +117,15 @@ Constraint_list *copy_constraint_list (Constraint_list *CL, int mode)
       NCL->copy_mode=mode;
       if (mode==SOFT_COPY)NCL->pCL=CL;
       
-
       NCL->STRUC_LIST=(mode==HARD_COPY)?duplicate_sequence (CL->STRUC_LIST):CL->STRUC_LIST;
       sprintf ( NCL->align_pdb_param_file, "%s", CL->align_pdb_param_file);
       sprintf ( NCL->align_pdb_hasch_mode, "%s", CL->align_pdb_hasch_mode);
 
       
       NCL->W=(mode==HARD_COPY)?duplicate_weights (CL->W):CL->W;
-      NCL->similarity_matrix=(mode==HARD_COPY)?duplicate_int (CL->similarity_matrix, -1, -1):CL->similarity_matrix;
-      NCL->score_similarity_matrix=(mode==HARD_COPY)?duplicate_int (CL->score_similarity_matrix, -1, -1):CL->score_similarity_matrix;
-      NCL->distance_matrix=(mode==HARD_COPY)?duplicate_int (CL->distance_matrix, -1, -1):CL->distance_matrix;
+      NCL->DM=(mode==HARD_COPY)?duplicate_distance_matrix (CL->DM):CL->DM;
+      NCL->ktupDM=(mode==HARD_COPY)?duplicate_distance_matrix (CL->ktupDM):CL->ktupDM;
+      NCL->RunName=CL->RunName;
       
       if (  mode==HARD_COPY && CL->translation){NCL->translation=vcalloc ((CL->S)->nseq, sizeof (int)); for ( a=0; a< (CL->S)->nseq; a++)NCL->translation[a]=CL->translation[a];}
       else{NCL->translation=CL->translation;}
@@ -227,6 +162,7 @@ Constraint_list *copy_constraint_list (Constraint_list *CL, int mode)
 
     /*Normalisation information*/
       NCL->normalise=CL->normalise;
+      NCL->overweight=CL->overweight;
       NCL->max_ext_value=CL->max_ext_value;
       NCL->max_value=CL->max_value;
     
@@ -256,6 +192,10 @@ Constraint_list *copy_constraint_list (Constraint_list *CL, int mode)
       NCL->sw_min_dist=CL->sw_min_dist;
       NCL->matrices_list=(mode==HARD_COPY)?duplicate_char (CL->matrices_list, -1, -1):CL->matrices_list;
       NCL->n_matrices=CL->n_matrices;
+      
+      sprintf (NCL->distance_matrix_mode, "%s", CL->distance_matrix_mode);
+      sprintf (NCL->distance_matrix_sim_mode, "%s", CL->distance_matrix_sim_mode);
+
       sprintf (NCL->tree_mode, "%s", CL->tree_mode);
       NCL->tree_aln=(mode==HARD_COPY)?copy_aln (CL->tree_aln, NULL):CL->tree_aln;
     /*Functions used for dynamic programming and Evaluation*/ 
@@ -318,13 +258,15 @@ Constraint_list *copy_constraint_list (Constraint_list *CL, int mode)
     /*Lookup table parameteres*/
       NCL->chunk= CL->chunk;
       /* Do NOT copy NCL->seq_indexed, NCL->start_index, NCL->max_L_len, NCL->chunk*/
-      /*if ( mode==SOFT_COPY)
+      /*
+	if ( mode==SOFT_COPY)
 	{
 	  NCL->seq_indexed=CL->seq_indexed;
 	  NCL->start_index=CL->start_index;
 	  NCL->end_index=CL->start_index;
 	  NCL->max_L_len=CL->max_L_len;
-	  }*/
+	  }
+      */
     /*PDB STRUCTURE ALIGNMENTS*/
       /* Do only a soft copy */
       if ( mode==SOFT_COPY)
@@ -349,33 +291,40 @@ Sequence *free_constraint_list (Constraint_list *CL)
     int a, b;
     Constraint_list *pCL;
 
-
+    
     /*Prepare the selective freeing of the CL data structure:
-      If the CL has been obtain from copy, every pointer that is identical to the parent CL (CL->pCL)
+      If the CL has been obtained from copy, every pointer that is identical to the parent CL (CL->pCL)
       will not be saved.
     */
-      
+   
+    
     if ( !CL)return NULL;
     else S=CL->S;
     
-    if ( CL->copy_mode==SOFT_COPY && !CL->pCL){vfree(CL); return S;}
+    if ( CL->copy_mode==SOFT_COPY && !CL->pCL)
+      {vfree(CL); return S;}
     else if ( CL->copy_mode==SOFT_COPY)
       {
+
 	pCL=CL->pCL;
 	CL->L=NULL;
+
 	if ( CL->M                      ==pCL->M                       )CL->M=NULL;
-	/*if (CL->start_index             ==pCL->start_index             )pCL->start_index=NULL;
-	if (CL->residue_index           ==pCL->residue_index           )CL->residue_index=NULL;
-	*/
 	
+	if (CL->start_index             ==pCL->start_index             )CL->start_index=NULL;
+	if (CL->end_index             ==pCL->end_index                 )CL->end_index=NULL;
+	if (CL->residue_index           ==pCL->residue_index           )CL->residue_index=NULL;
+
 	if ( CL->fp                     ==pCL->fp                      )CL->fp=NULL;
 	if ( CL->matrices_list          ==pCL->matrices_list           )CL->matrices_list=NULL;
+
 	
 	if ( CL->STRUC_LIST             ==pCL->STRUC_LIST              )CL->STRUC_LIST=NULL;
 	if ( CL->W                      ==pCL->W                       )CL->W=NULL;
-	if ( CL->similarity_matrix      ==pCL->similarity_matrix       )CL->similarity_matrix=NULL;
-	if ( CL->score_similarity_matrix==pCL->score_similarity_matrix )CL->score_similarity_matrix=NULL;
-	if ( CL->distance_matrix        ==pCL->distance_matrix         )CL->distance_matrix=NULL;
+	if ( CL->DM                     ==pCL->DM                      )CL->DM=NULL;
+	if ( CL->ktupDM                 ==pCL->ktupDM                      )CL->ktupDM=NULL;
+
+
 	if ( CL->translation            ==pCL->translation             )CL->translation=NULL;
 	if ( CL->moca                   ==pCL->moca                    )CL->moca=NULL;
 	if ( CL->Prot_Blast             ==pCL->Prot_Blast              )CL->Prot_Blast=NULL;
@@ -385,15 +334,20 @@ Sequence *free_constraint_list (Constraint_list *CL)
 	
       }
 
+    
     /*End of selective freeing of the CL data structure*/
 
 
+    
     if ( CL->L)free_int (CL->L, -1);
     if ( CL->M)free_int (CL->M, -1);
     if ( CL->fp)vfclose (CL->fp);
     if ( CL->matrices_list)free_char(CL->matrices_list,-1);
+   
+
     if ( CL->start_index)free_int ( CL->start_index,-1);
     if ( CL->end_index)free_int ( CL->end_index,-1);
+    
     if ( CL->residue_index)
       {
 	for ( a=0; a< (CL->S)->nseq; a++)
@@ -404,13 +358,13 @@ Sequence *free_constraint_list (Constraint_list *CL)
 	  }
 	vfree(CL->residue_index);
       }
+       
     
- 
     if ( CL->STRUC_LIST)free_sequence ( CL->STRUC_LIST, (CL->STRUC_LIST)->nseq);
     if ( CL->W)free_weights (CL->W);
-    if ( CL->similarity_matrix) free_int (CL->similarity_matrix, -1);
-    if ( CL->score_similarity_matrix) free_int (CL->score_similarity_matrix, -1);
-    if ( CL->distance_matrix) free_int (CL->distance_matrix, -1);
+    
+    CL->DM=free_distance_matrix (CL->DM);
+    CL->ktupDM=free_distance_matrix (CL->ktupDM);
     
     if ( CL->translation)vfree(CL->translation);
     if ( CL->moca)free_moca (CL->moca);
@@ -419,11 +373,32 @@ Sequence *free_constraint_list (Constraint_list *CL)
     if ( CL->Pdb_Blast) free_blast_param ( CL->Pdb_Blast);
        
     if (CL->seq_for_quadruplet)vfree (CL->seq_for_quadruplet);
-    
+
     vfree(CL);
     return S;
     }
 
+Distance_matrix * free_distance_matrix ( Distance_matrix *DM)
+{
+  if (!DM)return NULL;
+  free_int ( DM->similarity_matrix,-1);
+  free_int ( DM->distance_matrix,-1);
+  free_int ( DM->score_similarity_matrix,-1);
+  vfree (DM);
+  return NULL;
+}
+Distance_matrix * duplicate_distance_matrix ( Distance_matrix *DMin)
+{
+  Distance_matrix *DM;
+  if (!DMin) return NULL;
+  
+  DM=vcalloc ( 1, sizeof (Distance_matrix));
+  DM->similarity_matrix=duplicate_int ( DMin->similarity_matrix, -1, -1);
+  DM->distance_matrix=duplicate_int ( DMin->distance_matrix, -1, -1);
+  DM->score_similarity_matrix=duplicate_int ( DMin->score_similarity_matrix, -1, -1);
+  return DM;
+}
+  
 /************************************************************************/
 /*                                                                      */
 /*            MOCA Functions                                            */
@@ -525,12 +500,12 @@ Sequence * declare_sequence ( int min, int max, int nseq)
 
     LS=vcalloc (1, sizeof ( Sequence));
 
-    LS->seq_comment=declare_char ( nseq,STRING+1);
-    LS->aln_comment=declare_char ( nseq,STRING+1);
+    LS->seq_comment=declare_char ( nseq,COMMENT_SIZE);
+    LS->aln_comment=declare_char ( nseq,COMMENT_SIZE);
     
     LS->file=declare_char( nseq,STRING+1);
     LS->seq=declare_char ( nseq, max+1);
-    LS->name=declare_char( nseq,STRING+1);
+    LS->name=declare_char( nseq,MAXNAMES+1);
     
     LS->len=vcalloc ( nseq, sizeof (int));
     LS->max_len=max;
@@ -543,17 +518,17 @@ Sequence * declare_sequence ( int min, int max, int nseq)
     }
 Sequence * realloc_sequence   (Sequence *OUT, int new_nseq, int max_len)
     {
-      int a;
+  
 
       if ( new_nseq<OUT->max_nseq)return OUT;
 
       OUT->min_len =MIN(OUT->min_len,max_len);
       OUT->max_len =MAX(OUT->max_len,max_len);
-      OUT->seq_comment =new_realloc_char ( OUT->seq_comment, new_nseq,STRING+1);
-      OUT->aln_comment =new_realloc_char ( OUT->aln_comment, new_nseq,STRING+1);
+      OUT->seq_comment =new_realloc_char ( OUT->seq_comment, new_nseq,COMMENT_SIZE);
+      OUT->aln_comment =new_realloc_char ( OUT->aln_comment, new_nseq,COMMENT_SIZE);
       
       OUT->seq     =new_realloc_char ( OUT->seq,     new_nseq,OUT->max_len+1);
-      OUT->name    =new_realloc_char ( OUT->name,    new_nseq,STRING+1);
+      OUT->name    =new_realloc_char ( OUT->name,    new_nseq,MAXNAMES+1);
       
       
       OUT->file    =new_realloc_char ( OUT->file,    new_nseq,STRING+1);
@@ -600,7 +575,7 @@ Sequence * duplicate_sequence (Sequence *S )
 
 void free_sequence ( Sequence *LS, int nseq)
 	{
-	int a;
+
 	
 	if ( !LS) return;
 
@@ -710,7 +685,12 @@ Alignment* copy_aln ( Alignment *A, Alignment *B)
 	   
 	   
 /*sequence Information*/	
-
+	    if ( A->generic_comment)
+	      {
+		vfree(B->generic_comment);
+		B->generic_comment=vcalloc (strlen(A->generic_comment)+1, sizeof (char));
+		sprintf ( B->generic_comment, "%s", A->generic_comment);
+	      }
 	    if ( (A->S)==NULL){vfree (B->len); B->len=vcalloc ( A->max_n_seq, sizeof (int));}
 	    ga_memcpy_int ( A->len, B->len, B->nseq);
 	    
@@ -762,6 +742,7 @@ Alignment* copy_aln ( Alignment *A, Alignment *B)
 /*Output Options*/    
 	    B->output_res_num=A->output_res_num;
 	    B->residue_case=A->residue_case;
+	    B->expand=A->expand;
 	    
 	    B->CL=A->CL;
 	    B->random_tag=A->random_tag;
@@ -791,6 +772,10 @@ Alignment* shrink_aln ( Alignment *A, int nseq, int *list)
 	    sprintf ( A->seq_al [a], "%s",B->seq_al [seq]);
 	    A->order[a][0]=B->order[seq][0];
 	    A->order[a][1]=B->order[seq][1];
+	    A->order[a][2]=B->order[seq][2];
+	    A->order[a][3]=B->order[seq][3];
+	    A->order[a][4]=B->order[seq][4];
+	    
 	    A->score_seq[a]=B->score_seq[seq];
 	    A->len[a]=B->len[seq];
 	    }
@@ -828,6 +813,10 @@ Alignment* extract_sub_aln ( Alignment *B, int nseq, int *list)
 	    for (b=0; b<=B->len_aln; b++)A->seq_al [a][b]=B->seq_al [seq][b];
 	    A->order[a][0]=B->order[seq][0];
 	    A->order[a][1]=B->order[seq][1];
+	    A->order[a][2]=B->order[seq][2];
+	    A->order[a][3]=B->order[seq][3];
+	    A->order[a][4]=B->order[seq][4];
+	    
 	    A->score_seq[a]=B->score_seq[seq];
 	    A->len[a]=B->len[seq];
 	    }
@@ -878,16 +867,23 @@ Alignment *declare_Alignment ( Sequence *S)
 	    }
 	LA->S=S;
 	
-	LA->seq_comment=declare_char_nomemset (LA->max_n_seq, STRING);
-	LA->aln_comment=declare_char_nomemset (LA->max_n_seq, STRING);
 	
+	LA->seq_comment=declare_char (LA->max_n_seq, COMMENT_SIZE);
+	LA->aln_comment=declare_char (LA->max_n_seq, COMMENT_SIZE);
+	
+
 	LA->seq_al=declare_char ( LA->max_n_seq,LA->declared_len );
-	LA->name=declare_char_nomemset (LA->max_n_seq, STRING);
+	LA->name=declare_char (LA->max_n_seq, MAXNAMES+1);
 	
 	
 	LA->file=declare_char (LA->max_n_seq, STRING);
 	LA->tree_order=declare_char (LA->max_n_seq, STRING);
-	LA->order= declare_int (LA->max_n_seq , 2);
+	LA->order= declare_int (LA->max_n_seq , 5);
+	//order[a][0]: sequence index in S
+	//order[a][1]: offset of the sequence
+	//order[a][2]: used by sw_gotoh_pair_wise
+	//order[a][3]: used by sw_gotoh_pair_wise
+	//order[a][4]: weight, -1
 	LA->score_seq= vcalloc (LA->max_n_seq, sizeof (int));
 
 	for ( a=0; a< LA->max_n_seq; a++)LA->order[a][0]=a;
@@ -895,7 +891,7 @@ Alignment *declare_Alignment ( Sequence *S)
 	LA->len_aln=0;
 	LA->score_aln=0;
 	LA->len=vcalloc (LA->max_n_seq, sizeof (int));		
-	
+		
 	if (S && S->name)for ( a=0; a<S->nseq; a++)
 	  {
 	    sprintf ( LA->name[a], "%s", S->name[a]);
@@ -911,7 +907,6 @@ Alignment * realloc_alignment ( Alignment *A, int new_len)
 	if (A==NULL)A=declare_Alignment (NULL);
 
 	return realloc_alignment2( A, A->max_n_seq,new_len);
-
 	}
 
 Alignment * realloc_aln2 ( Alignment *A, int n_nseq, int n_len){return realloc_alignment2(A, n_nseq, n_len);}
@@ -944,6 +939,7 @@ Alignment * realloc_alignment2 ( Alignment *A, int n_nseq, int n_len)
 	    {	    
 	        A->len          =vrealloc( A->len      , sizeof (int)*n_nseq);
 		for (a=nseq; a< n_nseq; a++)A->len[a]=0;
+				
 		A->declared_len =n_len;
 		A->max_n_seq    =n_nseq;
 
@@ -990,6 +986,7 @@ Sequence* free_Alignment ( Alignment *LA)
 	  free_char ( LA->name,-1);
 	  
 	  free_char ( LA->tree_order,-1);	
+	  vfree ( LA->generic_comment);
 	  free_char ( LA->seq_comment, -1);
 	  free_char ( LA->aln_comment, -1);
 	  
@@ -997,6 +994,7 @@ Sequence* free_Alignment ( Alignment *LA)
 	  
 	  vfree ( LA->score_seq);
 	  vfree ( LA->len);
+	  
 	  free_profile (LA->P);
 	  if ( LA->A){free_Alignment (LA->A);LA->A=NULL;}
 	  
@@ -1005,6 +1003,15 @@ Sequence* free_Alignment ( Alignment *LA)
 	  return S;
 	}
 
+Alignment * update_aln_random_tag ( Alignment *A)
+{
+  static int tag;
+  if ( !A) return A;
+  
+  A->random_tag=++tag;
+  return A;
+}
+  
 Profile   *copy_profile   (Profile *P1)
 {
 
@@ -1065,7 +1072,7 @@ Memcontrol *memlast;
 
 FILE* print_mem_usage (FILE *fp, char *comment)
 {
-  fprintf ( fp, "\n%s Memory Usage: Current= %.3f Mb, Max= %.3f Mb\n", comment,(float)((float)alloc_mem/(1024*1024)),(float)((float)tot_mem/(1024*1024)) );
+  fprintf ( fp, "# %s Memory Usage: Current= %.3f Mb, Max= %.3f Mb\n", comment,(float)((float)alloc_mem/(1024*1024)),(float)((float)tot_mem/(1024*1024)) );
   return fp;
 }
 void set_max_mem (int m)
@@ -1084,7 +1091,7 @@ int verify_memory (int s)
       fprintf (stderr, "\n%s Requires Too Much Memory: %d Megabytes [FATAL:%s]\n", PROGRAM,(int)(alloc_mem/1024*1024),PROGRAM);
       fprintf (stderr, "Tip: Rerun your Job with a smaller dataset\n");
       
-      exit (EXIT_FAILURE);
+      myexit (EXIT_FAILURE);
     }
   else
     return 1;
@@ -1163,7 +1170,7 @@ void *sub_vcalloc ( size_t nobj, size_t size, int MODE)
 	{
 	void *x;
 	Memcontrol *M;
-	static int set, unset, count;
+	
 	if ( nobj<=0 || size<=0)return NULL;/*crash ("\n0 bytes in vmalloc\n");*/
 	else x=vmalloc (nobj*size);
 
@@ -1346,7 +1353,7 @@ void * declare_arrayNnomemset (int ndim, size_t size, ...)
   int *array;
   void **p;
   int a;
-
+  
   va_start (ap, size);
   
   array=vcalloc (ndim, sizeof (int));
@@ -1360,10 +1367,12 @@ void * declare_arrayNnomemset (int ndim, size_t size, ...)
 
   if ( ndim==2)
     {
-      int t;
+      
       p=vcalloc_nomemset (array[0], sizeof ( void*));
       for (a=0; a< array[0]; a++)
+	{
 	p[a]=vcalloc_nomemset (array[1], size);
+	}
     }
   else
     {
@@ -1412,7 +1421,7 @@ void * declare_arrayN (int ndim, size_t size, ...)
 
   if ( ndim==2)
     {
-      int t;
+
       p=vcalloc_nomemset (array[0], sizeof ( void*));
       for (a=0; a< array[0]; a++)
 	p[a]=vcalloc (array[1], size);
@@ -1547,24 +1556,36 @@ void **realloc_arrayN2 ( int ndim, void ** p, int *A, size_t size)
 }
 
 
+
 void ** realloc_array (void **array,size_t size, int first, int second, int ext1, int ext2)
 {
   int a;
+  int d1, d2;
   if ( array==NULL)return declare_array (((first==-1)?0:first)+ext1, ((second==-1)?0:second)+ext2, size);
   else if ( first==-1)
     {
       first=read_array_size (array, sizeof (void *));
     }
-  for ( a=(first+ext1); a<first; a++)vfree (array[a]);
-  
-  
-  array=vrealloc (array, sizeof (void*)*(ext1+first));
   if (second==-1)second=read_array_size(array[0], size);
-  for (a=0; a< (first+ext1); a++)
-    array[a]=vrealloc ( array[a], size*(second+ext2));
+  
+  d1=first+ext1;
+  d2=second+ext2;
+
+  for ( a=d1; a<first; a++)vfree (array[a]);
+  array=vrealloc (array, sizeof (void*)*d1);
+  
+  if ( d2!=second)
+    {
+      for (a=0; a<d1 && a<first; a++)
+	array[a]=vrealloc ( array[a], size*d2);
+    }
+  for ( a=first; a< d1; a++)
+    array[a]=vrealloc ( array[a], size*d2);
   return array;
 }
 
+
+
   
 
 #define REALLOC_ARRAY(type,wf,rf,function1,function2,function3)\
@@ -1601,6 +1622,7 @@ type ** function1 ( type **array, int ext1, int ext2)\
      function3(array, -1);\
      return new_array;\
     }
+
 NEW_REALLOC_ARRAY(short,write_size_short,read_size_short,new_realloc_short,declare_short,free_short)
 NEW_REALLOC_ARRAY(char,write_size_char,read_size_char,new_realloc_char,declare_char,free_char)
 NEW_REALLOC_ARRAY(int,write_size_int,read_size_int,new_realloc_int,declare_int,free_int)
@@ -1664,14 +1686,17 @@ Alignment ** free_aln_array (Alignment **array)
    return  NULL;
    }
 
-Fname *declare_fname ()
+Fname *declare_fname (int size)
    {
    Fname *F;
 
+   size+=strlen (get_home_4_tcoffee())+FILENAMELEN+1;
+  
    F=vcalloc ( 1, sizeof (Fname));
-   F->name  =vcalloc ( FILENAMELEN, sizeof (char));
-   F->path  =vcalloc ( FILENAMELEN, sizeof (char));
-   F->suffix=vcalloc ( FILENAMELEN, sizeof (char));
+   F->name  =vcalloc ( size, sizeof (char));
+   F->path  =vcalloc ( size, sizeof (char));
+   F->suffix=vcalloc ( size, sizeof (char));
+   F->full=vcalloc ( size, sizeof (char));
    return F;
    }
 
@@ -1686,7 +1711,7 @@ Fname *free_fname ( Fname *F)
 /* Centre National de la Recherche Scientifique (CNRS) */
 /*and */
 /*Cedric Notredame */
-/*Tue May 10 12:08:44     2005. */
+/*Fri Oct 26 17:03:04     2007. */
 /*All rights reserved.*/
 /*This file is part of T-COFFEE.*/
 /**/
