@@ -151,7 +151,7 @@ Alignment * extract_domain ( Constraint_list *CL)
      else if ( !(CL->moca)->moca_len)
        {
 	 analyse_sequence (CL);
-	 exit (1);
+	 myexit (EXIT_FAILURE);
        }
 
      /*CASE 2: Automatic Domain Extraction: Find Coordinates*/
@@ -177,7 +177,7 @@ Alignment * extract_domain ( Constraint_list *CL)
      C=add_seq2aln (CL,C, CL->S);
      print_aln (C);
      
-     exit (1);
+     myexit (EXIT_FAILURE);
      
      while ( step>0)
        {
@@ -193,7 +193,7 @@ Alignment * extract_domain ( Constraint_list *CL)
      C->output_res_num=1;
      print_aln (C);
 
-     exit (1);
+     myexit (EXIT_FAILURE);
      return C;
 
 
@@ -203,7 +203,7 @@ Alignment * interactive_domain_extraction ( Constraint_list *CL)
      int LEN=0;
      int START=1;
      int SCALE=2;
-     int GOP=3;
+     int GOPP=3;
 
      int iteration=0;
      char *choice;
@@ -226,7 +226,7 @@ Alignment * interactive_domain_extraction ( Constraint_list *CL)
      parameters[0][START]=(CL->moca)->moca_start;
      parameters[0][LEN]=  (CL->moca)->moca_len;
      parameters[0][SCALE]=(CL->moca)->moca_scale;
-     parameters[0][GOP]=CL->gop;
+     parameters[0][GOPP]=CL->gop;
      iteration=0;
      sprintf ( last_start, "%d", (CL->moca)->moca_start);
      sprintf ( out_format, "mocca_aln");
@@ -248,7 +248,7 @@ Alignment * interactive_domain_extraction ( Constraint_list *CL)
 		 parameters[iteration][START]=parameters[iteration-1][START];
 		 parameters[iteration][LEN]=parameters[iteration-1][LEN];
 		 parameters[iteration][SCALE]=parameters[iteration-1][SCALE];
-		 parameters[iteration][GOP]=parameters[iteration-1][GOP];
+		 parameters[iteration][GOPP]=parameters[iteration-1][GOPP];
 
 		 if ( c=='>')parameters[iteration][LEN]=atoi(choice+1);
 		 else if ( c=='|')
@@ -283,8 +283,8 @@ Alignment * interactive_domain_extraction ( Constraint_list *CL)
 		 else if ( c=='C'||c=='c')parameters[iteration][SCALE]=atoi(choice+1);
 		 else if ( c=='G'||c=='g')
 		   {
-		     parameters[iteration][GOP]=atoi(choice+1);
-		     CL->gop=parameters[iteration][GOP];
+		     parameters[iteration][GOPP]=atoi(choice+1);
+		     CL->gop=parameters[iteration][GOPP];
 		   }
 		 else if (  c=='F'||c=='f')
 		   {
@@ -305,6 +305,9 @@ Alignment * interactive_domain_extraction ( Constraint_list *CL)
 			 parameters[iteration][SCALE]=0;
 		       }
 		     
+		     (CL->moca)->moca_scale=parameters[iteration][SCALE];
+		      CL->gop=parameters[iteration][GOPP];
+
 		     C=extract_domain_with_coordinates (C,parameters[iteration][START],parameters[iteration][LEN],CL);
 		     
 		     if ( C==NULL)
@@ -343,7 +346,7 @@ Alignment * interactive_domain_extraction ( Constraint_list *CL)
 		       }
 		   }
 		
-		 fprintf ( stderr, "\t[ITERATION %3d][START=%s][LEN=%3d][GOP=%3d][SCALE=%4d]\t",iteration,last_start,parameters[iteration][LEN],parameters[iteration][GOP],parameters[iteration][SCALE]);
+		 fprintf ( stderr, "\t[ITERATION %3d][START=%s][LEN=%3d][GOPP=%3d][SCALE=%4d]\t",iteration,last_start,parameters[iteration][LEN],parameters[iteration][GOPP],parameters[iteration][SCALE]);
 		 a=0;
 		 fprintf ( stderr, "Your Choice: ");
 		 while ( (c=fgetc(stdin))!='\n')choice[a++]=c;
@@ -351,7 +354,7 @@ Alignment * interactive_domain_extraction ( Constraint_list *CL)
 		 }
 	     }
      
-     if (!RESULT)exit(1);
+     if (!RESULT)myexit(EXIT_SUCCESS);
      if ( RESULT)RESULT->output_res_num=0;
      return RESULT;
    }
@@ -587,7 +590,7 @@ int * analyse_sequence ( Constraint_list *CL)
  int a, p;
  int len, start, n_dots;
  int left, right, tw, r, w;
- int best_tw, best_start, best_len;
+ int best_tw, best_start=0, best_len=0;
  int l;
  int max_len=200;
  
@@ -651,7 +654,7 @@ int * analyse_sequence ( Constraint_list *CL)
 /* Centre National de la Recherche Scientifique (CNRS) */
 /*and */
 /*Cedric Notredame */
-/*Tue May 10 12:08:44     2005. */
+/*Fri Oct 26 17:03:04     2007. */
 /*All rights reserved.*/
 /*This file is part of T-COFFEE.*/
 /**/
