@@ -363,7 +363,7 @@ int make_fasta_cdna_pair_wise (Alignment *B,Alignment *A,int*in_ns, int **l_s,Co
       static Dp_Model  *M;
       int l0, l1;
       int len_i, len_j;
-      int f0, f1;
+      int f0=0, f1=0;
       int deltaf0, deltaf1, delta;
       int nr1, nr2;
       int ala, alb, aa0, aa1;
@@ -599,7 +599,7 @@ Dp_Result * make_fast_dp_pair_wise (Alignment *A,int*ns, int **l_s, Constraint_l
 	  int a, i, j,l;
 	  int state, cur_state, prev_state;
 	  int pos_i,  pos_j;
-	  int last_i, last_j;
+	  int last_i=0, last_j=0;
 	  int prev_i, prev_j;
 	  int len_i, len_j, len;
 	  int t, e, em;
@@ -611,7 +611,7 @@ Dp_Result * make_fast_dp_pair_wise (Alignment *A,int*ns, int **l_s, Constraint_l
 	  int model_index;
 	  /*TRACEBACK*/
 	  Dp_Result *DPR;
-	  int k, next_k;
+	  int k=0, next_k;
 	  int new_i, new_j;
 	  
 	  
@@ -947,7 +947,7 @@ int ** evaluate_diagonals_cdna ( Alignment *B, int *ns, int **l_s, Constraint_li
 /* Centre National de la Recherche Scientifique (CNRS) */
 /*and */
 /*Cedric Notredame */
-/*Tue May 10 12:08:44     2005. */
+/*Fri Oct 26 17:03:04     2007. */
 /*All rights reserved.*/
 /*This file is part of T-COFFEE.*/
 /**/
