@@ -19,6 +19,10 @@ Alignment *clean_maln ( Alignment *A, Alignment *I, int T, int n_it)
     int **segment_list;
     
         
+    add_warning ( stderr, "\nWARNING: -clean_aln is not supported anymore [PROGRAM:%s]\n", PROGRAM);
+    return A;
+
+    
    
     /*Initialization*/
     a=0;
@@ -90,7 +94,7 @@ Alignment *clean_maln ( Alignment *A, Alignment *I, int T, int n_it)
       {
 	for ( a=0; a< n_segment; a++)
 	  {
-
+	    HERE ("1");
 	    A=realign_segment ( segment_list[a][0], segment_list[a][1], segment_list[a][2], A, C);
 	   
 	  }
@@ -283,7 +287,7 @@ Dp_Model * initialize_seg2prf_model(int left_tg_mode, int right_tg_mode, Constra
     M->model_properties[0][M->DELTA_I]=-1;
     M->model_properties[0][M->DELTA_J]= 0;	
 
-    M->model_emission_function[0][M->EMISSION]      =very_fast_get_dp_cost;
+    M->model_emission_function[0][M->EMISSION]      =cw_profile_get_dp_cost;
     M->model_emission_function[0][M->START_EMISSION]=get_start_gep_cost;
     M->model_emission_function[0][M->TERM_EMISSION] =get_start_gep_cost;
    
@@ -353,7 +357,7 @@ int get_term_gep_cost (Alignment *A, int**pos1, int ns1, int*list1, int col1, in
 /* Centre National de la Recherche Scientifique (CNRS) */
 /*and */
 /*Cedric Notredame */
-/*Tue May 10 12:08:44     2005. */
+/*Fri Oct 26 17:03:04     2007. */
 /*All rights reserved.*/
 /*This file is part of T-COFFEE.*/
 /**/
