@@ -2,12 +2,215 @@
 #include <stdlib.h>
 #include <math.h>
 #include <stdarg.h>
-
+#include <string.h>
+#include <ctype.h>
 
 #include "io_lib_header.h"
 #include "util_lib_header.h"
 #include "define_header.h"
 #include "dp_lib_header.h"
+
+
+int commonsextet( short *table, int *pointt );
+void makecompositiontable( short *table, int *pointt );
+int *code_seq (char *seq, char *type);
+int * makepointtable( int *pointt, int *n, int ktup );
+
+static int tsize;
+int commonsextet( short *table, int *pointt )
+{
+	int value = 0;
+	short tmp;
+	int point;
+	static short *memo = NULL;
+	static int *ct = NULL;
+	static int *cp;
+		
+	if( !memo )
+	{
+		memo = vcalloc( tsize, sizeof( short ) );
+		ct = vcalloc( tsize, sizeof( int ) );
+	}
+
+	cp = ct;
+	while( ( point = *pointt++ ) != END_ARRAY )
+	{
+	  tmp = memo[point]++;
+	  if( tmp < table[point] )
+	    value++;
+	  if( tmp == 0 ) 
+	    {
+	      *cp++ = point;
+	    }
+	}
+	*cp = END_ARRAY;
+	
+	cp =  ct;
+	while( *cp != END_ARRAY )
+		memo[*cp++] = 0;
+
+	return( value );
+}
+
+void makecompositiontable( short *table, int *pointt )
+{
+	int point;
+
+	while( ( point = *pointt++ ) != END_ARRAY )
+	  {
+	    table[point]++;
+	  }
+}
+int *code_seq (char *seq, char *type)
+{
+  static int *code;
+  static int *aa, ng;
+  int a, b, l;
+  
+  
+  if (!aa)
+    {
+      char **gl;
+      if ( strm (type, "DNA"))
+	{
+	  gl=declare_char (4,5);
+	  sprintf ( gl[ng++], "Aa");
+	  sprintf ( gl[ng++], "Gg");
+	  sprintf ( gl[ng++], "TtUu");
+	  sprintf ( gl[ng++], "Cc");
+	}
+      else
+	{
+     
+	  gl=make_group_aa ( &ng, "mafft");
+	}
+      aa=vcalloc ( 256, sizeof (int));
+      for ( a=0; a<ng; a++)
+	{
+	  for ( b=0; b< strlen (gl[a]); b++) 
+	    {
+	      aa[(int)gl[a][b]]=a;
+	    }
+	}
+      free_char (gl, -1);
+    }
+  
+  
+  l=strlen (seq);
+  
+  if ( code) code--;
+  
+  if ( !code || read_array_size (code, sizeof (int))<(l+2))
+    {
+      vfree (code);
+      code=vcalloc (l+2, sizeof (int));
+    }
+  code[0]=ng;
+  code++;
+  for (a=0; a<l; a++)
+    {
+      code[a]=aa[(int)seq[a]];
+    }
+  
+  code[a]=END_ARRAY;
+  return code;
+}
+    
+int * makepointtable( int *pointt, int *n, int ktup )
+{
+  int point, a, ng;
+  register int *p;
+  static int *prod;
+
+  ng=n[-1];
+  
+  if (!prod)
+    {
+      prod=vcalloc ( ktup, sizeof (int));
+      for ( a=0; a<ktup; a++)
+	{
+	  prod[ktup-a-1]=(int)pow(n[-1],a);
+	}
+    }
+  p = n;
+  
+  for (point=0,a=0; a<ktup; a++)
+    {
+      point+= *n++ *prod[a];
+    }
+  
+  *pointt++ = point;
+  
+  while( *n != END_ARRAY )
+    {
+      point -= *p++ * prod[0];
+      point *= ng;
+      point += *n++;
+      *pointt++ = point;
+    }
+  *pointt = END_ARRAY;
+  return pointt;
+}
+
+
+int ** ktup_dist_mat ( char **seq, int nseq, int ktup, char *type)
+{
+  //Adapted from MAFFT 5: fast ktup
+  int **pointt,*code=NULL, **pscore;
+  int i, l, j;
+  
+  double **mtx, score0;
+  
+
+  for (l=0,i=0;i<nseq; i++)
+    {
+      l=MAX(l, strlen(seq[i]));
+    }
+
+  pointt=declare_int (nseq, l);
+  mtx=declare_double (nseq, nseq);
+  pscore=declare_int ( nseq, nseq);
+  
+  //Make sure ktup is not longer than the shortest sequence
+  for (i=0; i<nseq; i++)
+    {
+      ungap(seq [i]);
+      ktup=MIN(ktup, strlen (seq[i]));
+    }
+  
+  for( i=0; i<nseq; i++ ) 
+    {
+      makepointtable( pointt[i], code=code_seq (seq[i], type),ktup);
+    }
+  tsize=(int)pow(code[-1], ktup);
+ 
+  for ( i=0; i<nseq; i++)
+    {
+      short *table1;
+      table1=vcalloc ( tsize,sizeof (short));
+      makecompositiontable( table1, pointt[i]);
+      
+      for (j=i; j<nseq; j++)
+	{
+	  mtx[i][j] = commonsextet( table1, pointt[j] );
+	}
+    }
+
+  for( i=0; i<nseq; i++ )
+    {
+      score0 = mtx[i][i];
+      for( j=0; j<nseq; j++ ) 
+	pscore[i][j] = (int)( ( score0 - mtx[MIN(i,j)][MAX(i,j)] ) / score0 * 3 * 10.0 + 0.5 );
+    }
+  for( i=0; i<nseq-1; i++ ) 
+    for( j=i+1; j<nseq; j++ ) 
+      {
+	pscore[i][j] = pscore[j][i]=100-MIN( pscore[i][j], pscore[j][i] );
+      }
+    return pscore;
+}
+ 
+
 int ** evaluate_diagonals_with_ktup_1 ( Alignment *A, int *ns, int **l_s, Constraint_list *CL,int maximise,int n_groups, char **group_list, int ktup);
 int ** evaluate_diagonals_with_ktup_2 ( Alignment *A, int *ns, int **l_s, Constraint_list *CL,int maximise,int n_groups, char **group_list, int ktup);
 
@@ -260,7 +463,7 @@ int ** evaluate_diagonals_with_clist ( Alignment *A, int *ns, int **l_s, Constra
     Weight the diagonals with the values read in the constraint list        
    */
     
-    int l1, l2,n_diag, s1, s2, r1, r2;
+    int l1, l2,n_diag, s1, s2, r1=0, r2=0;
     int a, b, c, d;
     int **diag;
     int **code;
@@ -392,8 +595,6 @@ int * extract_N_diag (int l1, int l2, int **sorted_diag, int n_chosen_diag, int
     slopes=vcalloc ( n_diag+1, sizeof (int));
  
     
-
-
     for ( a=n_diag; a>0 && a>(n_diag-n_chosen_diag); a--)
             {
 	    current_diag=sorted_diag[a][0];
@@ -454,14 +655,15 @@ int cfasta_gotoh_pair_wise (Alignment *A,int*ns, int **l_s,Constraint_list *CL)
 	int max_n_chosen_diag;
 	int l1, l2;
         /********Prepare Penalties******/
-		
+	
+
 	maximise=CL->maximise;
 	ktup=CL->ktup;
 
 	/********************************/
 	
 	
-	
+
 	
 	if ( !group_list)
 	   {
@@ -482,7 +684,8 @@ int cfasta_gotoh_pair_wise (Alignment *A,int*ns, int **l_s,Constraint_list *CL)
 	    {
 		step=CL->fasta_step;
 	    }
-	
+
+
 	tot_diag=evaluate_diagonals ( A, ns, l_s, CL, maximise,n_groups,group_list, ktup);
 
 
@@ -501,12 +704,13 @@ int cfasta_gotoh_pair_wise (Alignment *A,int*ns, int **l_s,Constraint_list *CL)
 	new_score=0;
 	vfree ( diag);
 
-	
- 	while (new_score!=score && n_chosen_diag< max_n_chosen_diag )
+
+	while (new_score!=score && n_chosen_diag< max_n_chosen_diag    )
 	  {
-	    
+
 	    
 	    score=new_score;
+	    
 	    ungap_sub_aln ( A, ns[0], l_s[0]);
 	    ungap_sub_aln ( A, ns[1], l_s[1]);
 	    
@@ -517,7 +721,9 @@ int cfasta_gotoh_pair_wise (Alignment *A,int*ns, int **l_s,Constraint_list *CL)
 	    
 	    diag     =extract_N_diag (strlen (A->seq_al[l_s[0][0]]),strlen (A->seq_al[l_s[1][0]]), tot_diag, n_chosen_diag, 0); 
 	    new_score=make_fasta_gotoh_pair_wise (  A, ns, l_s, CL, diag);
+	    
 	    vfree ( diag);
+
 	  }
 	
 	score=new_score;
@@ -553,7 +759,7 @@ int fasta_gotoh_pair_wise (Alignment *A,int*ns, int **l_s,Constraint_list *CL)
 	/********************************/
 	
 
-	
+
 	if ( !group_list)
 	   {
 	       group_list=make_group_aa (&n_groups, CL->matrix_for_aa_group);
@@ -631,7 +837,8 @@ int make_fasta_gotoh_pair_wise (Alignment *A,int*ns, int **l_s,Constraint_list *
 		
 /*VARIABLES FOR THE MULTIPLE SEQUENCE ALIGNMENT*/
 	int a, b,c,k, t;
-	int l1, l2,eg, ch, sub,score=0, fop,last_i, last_j, i, delta_i, j, pos_j, ala, alb, LEN, n_diag, match1, match2;
+	int l1, l2,eg, ch, sub,score=0, last_i=0, last_j=0, i, delta_i, j, pos_j, ala, alb, LEN, n_diag, match1, match2;
+	int su, in, de, tr;
 	
 	int **C, **D, **I, **trace, **pos0, **LD;
 	int lenal[2], len;
@@ -644,11 +851,9 @@ int make_fasta_gotoh_pair_wise (Alignment *A,int*ns, int **l_s,Constraint_list *
 	TG_MODE=CL->TG_MODE;
 	maximise=CL->maximise;
 	
-
+	
 	/********************************/
-	
-	
-	
+		
 
         n_diag=diag[0];
 
@@ -726,6 +931,7 @@ int make_fasta_gotoh_pair_wise (Alignment *A,int*ns, int **l_s,Constraint_list *
 			
 		for (eg=0, j=1; j<=n_diag; j++)
 		    {
+		      
 			pos_j=diag[j]-lenal[0]+i;
 			if (pos_j<=0 || pos_j>l2 )
 			    {
@@ -742,7 +948,6 @@ int make_fasta_gotoh_pair_wise (Alignment *A,int*ns, int **l_s,Constraint_list *
 			else eg=1;			
 			I[i][j]=best_of_a_b (I[i][j-1], C[i][j-1]+l_gop, maximise)+len*l_gep;
 			
-
 		    /*2 Identify the best deletion in S2*/
 			l_gop=(pos_j==lenal[1])?((TG_MODE==0)?gop:0):gop;
 			l_gep=(pos_j==lenal[1])?((TG_MODE==2)?0:gep):gep;
@@ -756,13 +961,37 @@ int make_fasta_gotoh_pair_wise (Alignment *A,int*ns, int **l_s,Constraint_list *
 
 
 			/*Identify the best way*/	
+			/*
 			score=C[i][j]=best_int ( 3, maximise, &fop, I[i][j], C[i-1][j]+sub, D[i][j]);
-			
-			
 			fop-=1;
 			if ( fop<0)trace[i][j]=fop*eg;
 			else if ( fop>0 ) {trace[i][j]=fop*LD[i][j];}
 			else if ( fop==0) trace[i][j]=0;
+			*/
+			
+			su=C[i-1][j]+sub;
+			in=I[i][j];
+			de=D[i][j];
+
+			/*HERE ("%d %d %d", su, in, de);*/
+			if (su>=in && su>=de)
+			  {
+			    score=su;
+			    tr=0;
+			  }
+			else if (in>=de) 
+			  {
+			    score=in;
+			    tr=-eg;
+			  }
+			else
+			  {
+			    score=de;
+			    tr=LD[i][j];
+			  }
+			trace[i][j]=tr;
+			C[i][j]=score;
+			
 
 			last_i=i;
 			last_j=j;
@@ -930,7 +1159,7 @@ int hasch_seq(char *seq, int **hs, int **lu,int ktup,char *alp)
 
 	for ( i=0; i< alp_size; i++)
 	    {
-		alp_lu[alp[i]]=i;
+	      alp_lu[(int)alp[i]]=i;
 	    }
 	
 	
@@ -941,7 +1170,7 @@ int hasch_seq(char *seq, int **hs, int **lu,int ktup,char *alp)
 	lu[0]=vcalloc ( limit+1, sizeof(int));
 	
 
-	if ( l==0)exit(1);
+	if ( l==0)myexit(EXIT_FAILURE);
 	
 	for (i=1;i<=ktup;i++)
            a[i] = (int) pow((double)(alp_size+1),(double)(i-1));
@@ -954,7 +1183,7 @@ int hasch_seq(char *seq, int **hs, int **lu,int ktup,char *alp)
 		for(j=1;j<=ktup;++j) 
 		   {
 		   if (is_gap(seq[i+j-2])){flag=TRUE;break;}
-		   else residue=alp_lu[seq[i+j-2]];
+		   else residue=alp_lu[(int)seq[i+j-2]];
 		   code+=residue*a[j];
 		   }
 
@@ -1078,19 +1307,96 @@ int precomputed_pair_wise (Alignment *A,int*ns, int **l_s,Constraint_list *CL)
       A->score=A->score_aln=id;
       return A->score;
     }
+int ktup_comparison_str ( char *seq1, char *seq2, const int ktup);
+int ktup_comparison_hasch ( char *i_seq1, char *i_seq2, const int ktup);
 int ktup_pair_wise (Alignment *A,int*ns, int **l_s,Constraint_list *CL)
     {
+      static char **gl;
+      static int ng;
+      char *seq1;
+      char *seq2;
+
+      int min_len=10;
 
       
-  
-      A->score=A->score_aln=ktup_comparison ( A->seq_al[l_s[0][0]], A->seq_al[l_s[1][0]], CL->ktup);
       
+      if ( !gl)
+	gl=make_group_aa (&ng, "vasiliky");
+      
+      
+      if ( ns[0]>1)seq1=sub_aln2cons_seq_mat (A, ns[0], l_s[0],"blosum62mt");
+      else 
+	{
+	  seq1=vcalloc ( strlen (A->seq_al[l_s[0][0]])+1, sizeof (char));
+	  sprintf ( seq1, "%s",A->seq_al[l_s[0][0]]);
+	}
+      if ( ns[1]>1)seq2=sub_aln2cons_seq_mat (A, ns[1], l_s[1],"blosum62mt");
+      else
+	{
+	  seq2=vcalloc ( strlen (A->seq_al[l_s[1][0]])+1, sizeof (char));
+	  sprintf ( seq2, "%s",A->seq_al[l_s[1][0]]);
+	}
+      
+      if ( strlen (seq1)<min_len || strlen (seq2)<min_len)
+	{
+	  Alignment *B;
+	  
+	  ungap(seq1); ungap(seq2);
+	  B=align_two_sequences ( seq1, seq2, "blosum62mt",-10, -1, "myers_miller_pair_wise");
+	  A->score=A->score_aln=aln2sim(B, "idmat");
+	  free_aln (B);
+	  return A->score;
+	}
+      else
+	{
+	  
+	  string_convert (seq1, ng, gl);
+	  string_convert (seq2, ng, gl);
+	  A->score=A->score_aln=ktup_comparison (seq1,seq2, CL->ktup);
+	}
+      
+      vfree (seq1); vfree (seq2);
       return A->score;
     }
+int ktup_comparison( char *seq2, char *seq1, const int ktup)
+{
+  return ktup_comparison_hasch ( seq2, seq1, ktup);
+}
+int ktup_comparison_str ( char *seq2, char *seq1, const int ktup)
+{
+  int a,l1, l2,c1, c2, end, start;
+  char *s1, *s2;
+  double score=0;
+  int max_dist=-1;
 
-
+  if ( max_dist==-1)max_dist=MAX((strlen (seq1)),(strlen (seq2)));
+  l1=strlen (seq1)-ktup;
+  l2=strlen (seq2);
   
-int ktup_comparison ( char *i_seq1, char *i_seq2, const int ktup)
+
+  for ( a=0; a< l1; a++)
+    {
+      c1=seq1[a+ktup];seq1[a+ktup]='\0';
+      s1=seq1+a;
+      
+      start=((a-max_dist)<0)?0:a-max_dist;
+      end=((a+max_dist)>=l2)?l2:a+max_dist;
+
+      c2=seq2[end];seq2[end]='\0';
+      s2=seq2+start;
+      
+      score+=(strstr(s2, s1)!=NULL)?1:0;
+      
+      seq1[a+ktup]=c1;
+      seq2[end]=c2;
+    }
+  score/=(l1==0)?1:l1;
+  score=((log(0.1+score)-log(0.1))/(log(1.1)-log(0.1)));
+  
+  return score*100;
+  
+}
+int ktup_comparison_hasch ( char *i_seq1, char *i_seq2, const int ktup)
 {
   /*Ktup comparison adapted from Rob Edgar, NAR, vol32, No1, 381, 2004*/
   /*1: hasch sequence 1
@@ -1099,17 +1405,18 @@ int ktup_comparison ( char *i_seq1, char *i_seq2, const int ktup)
   
   char c;
   int key;
-  
+
   static HaschT*H1;
   static char *pseq;
   Hasch_entry *e;  
   char *s;  
   int l, ls;
-  
+  int p, a, max_dist=-1;
   double score=0;
 
   
-  if (pseq!=i_seq1)
+  
+  if (!strm (i_seq1, pseq))
     {
       if (H1)
 	{
@@ -1117,28 +1424,33 @@ int ktup_comparison ( char *i_seq1, char *i_seq2, const int ktup)
 	  string2key (NULL, NULL);
 	}
       H1=hasch_sequence ( i_seq1, ktup);
-      pseq=i_seq1;
+      vfree (pseq);pseq=vcalloc ( strlen (i_seq1)+1, sizeof (char));
+      sprintf ( pseq, "%s", i_seq1);
     }
-  
-   
+
   ls=l=strlen (i_seq2);
   s=i_seq2;
-  while (ls>=ktup)
+  p=0;
+  while (ls>ktup)
     {
       c=s[ktup];s[ktup]='\0';
       key=string2key (s, NULL);
       e=hsearch (H1,key,FIND, declare_ktup_hasch_data, free_ktup_hasch_data);
-      score +=(e!=NULL);
-      s[ktup]=c;s++;ls--;
+      
+      if ( e==NULL);
+      else if ( max_dist==-1)score++;
+      else
+	{
+	  for ( a=1; a<=(e->data)->list[1]; a++)
+	    if (FABS((p-(e->data)->list[a]))<=max_dist)
+	      {score++; break;}
+	}
+      s[ktup]=c;s++;p++;ls--;
     }
-  score/=(l-ktup+1);
+  score/=(l-ktup);
+  score=(log(0.1+score)-log(0.1))/(log(1.1)-log(0.1));
   
-  /*
-    score=(score)/(l-ktup+1);
-    score=(log(0.1+score)-log(0.1))/(log(1.1)-log(0.1));
-  */
-  /*Kimura correction: makes %id additive
-    score=score-((1-(score*score))/5);*/
+  if ( score>100) score=100;
   return (int)(score*100);
 }
  
@@ -1206,12 +1518,13 @@ int ** evaluate_diagonals_with_ktup ( Alignment *A, int *ns, int **l_s, Constrai
 
   pos=aln2pos_simple ( A,-1, ns, l_s);
   
-  seq1=aln2cons_seq (A, ns[0], l_s[0], n_groups, group_list);
-  seq2=aln2cons_seq (A, ns[1], l_s[1], n_groups, group_list);
+  seq1=aln2cons_maj (A, ns[0], l_s[0], n_groups, group_list);
+  seq2=aln2cons_maj (A, ns[1], l_s[1], n_groups, group_list);
   l1=strlen (seq1);
   l2=strlen (seq2);
   n_diag=l1+l2-1;
   
+  
   diag=declare_int (n_diag+2, 3);
   for ( a=0; a<n_diag+2; a++)diag[a][0]=a;
   
@@ -1249,6 +1562,7 @@ int ** evaluate_diagonals_with_ktup ( Alignment *A, int *ns, int **l_s, Constrai
     }
   
   sort_int (diag+1, 2, 1,0,n_diag-1);
+  
   hdestroy (H1,declare_ktup_hasch_data, free_ktup_hasch_data); hdestroy (H2,declare_ktup_hasch_data, free_ktup_hasch_data);
   vfree (seq1); vfree (seq2);vfree (sb);free_int (pos, -1);
   return diag;
@@ -1358,7 +1672,7 @@ int ** evaluate_diagonals_with_ktup_1 ( Alignment *A, int *ns, int **l_s, Constr
 /* Centre National de la Recherche Scientifique (CNRS) */
 /*and */
 /*Cedric Notredame */
-/*Tue May 10 12:08:44     2005. */
+/*Fri Oct 26 17:03:04     2007. */
 /*All rights reserved.*/
 /*This file is part of T-COFFEE.*/
 /**/
