@@ -13,7 +13,7 @@
 int cfasta_gotoh_pair_wise_sw (Alignment *A,int*ns, int **l_s,Constraint_list *CL)
     {
       fprintf ( stderr, "\ncfasta_gotoh_pair_wise_sw not implemented yet\n");
-      exit (1);
+      myexit (EXIT_FAILURE);
       return 0;
       
     }
@@ -21,14 +21,14 @@ int cfasta_gotoh_pair_wise_sw (Alignment *A,int*ns, int **l_s,Constraint_list *C
 int fasta_gotoh_pair_wise_sw (Alignment *A,int*ns, int **l_s,Constraint_list *CL)
     {
      fprintf ( stderr, "\nfasta_gotoh_pair_wise_sw not implemented yet\n");
-      exit (1); 
+      myexit (EXIT_FAILURE); 
       return 0;
     }
 
 int make_fasta_gotoh_pair_wise_sw (Alignment *A,int*ns, int **l_s,Constraint_list *CL, int *diag)
     {
       fprintf ( stderr, "\nmake_fasta_gotoh_pair_wise_sw not implemented yet\n");
-      exit (1); 
+      myexit (EXIT_FAILURE); 
       return 0;
     }
 
@@ -37,7 +37,7 @@ int make_fasta_gotoh_pair_wise_sw (Alignment *A,int*ns, int **l_s,Constraint_lis
 /* Centre National de la Recherche Scientifique (CNRS) */
 /*and */
 /*Cedric Notredame */
-/*Tue May 10 12:08:44     2005. */
+/*Fri Oct 26 17:03:04     2007. */
 /*All rights reserved.*/
 /*This file is part of T-COFFEE.*/
 /**/
