@@ -39,8 +39,8 @@ Dp_Result * make_fast_generic_dp_pair_wise (Alignment *A, int*ns, int **l_s,Dp_M
 	  static int *Mat, *LMat, *trace;
 	  int a, i, j,l;
 	  int state, cur_state, prev_state;
-	  int pos_i,  pos_j;
-	  int last_i, last_j;
+	  int pos_i=0,  pos_j=0;
+	  int last_i=0, last_j=0;
 	  int prev_i, prev_j;
 	  int len_i, len_j, len;
 	  int t, e, em;
@@ -52,7 +52,7 @@ Dp_Result * make_fast_generic_dp_pair_wise (Alignment *A, int*ns, int **l_s,Dp_M
 	  int model_index;
 	  /*TRACEBACK*/
 	  Dp_Result *DPR;
-	  int k, next_k;
+	  int k=0, next_k;
 	  int new_i, new_j;
 	  
 	  
@@ -351,7 +351,7 @@ Dp_Result * free_dp_result (Dp_Result *D )
 /* Centre National de la Recherche Scientifique (CNRS) */
 /*and */
 /*Cedric Notredame */
-/*Tue May 10 12:08:44     2005. */
+/*Fri Oct 26 17:03:04     2007. */
 /*All rights reserved.*/
 /*This file is part of T-COFFEE.*/
 /**/
