@@ -2,7 +2,7 @@
 #include <stdlib.h>
 #include <math.h>
 #include <stdarg.h>
-
+#include <string.h>
 
 #include "io_lib_header.h"
 #include "util_lib_header.h"
@@ -163,9 +163,10 @@ int gotoh_pair_wise_sw (Alignment *A, int*ns, int **l_s,Constraint_list *CL)
 	  }
 	else
 	  {
-	    g=(CL->gop)*SCORE_K;
-	    h=(CL->gep)*SCORE_K;
+	    g=(CL->gop-CL->nomatch)*SCORE_K;
+	    h=(CL->gep-CL->nomatch)*SCORE_K;
 	  }
+	fprintf ( stderr, "\n%d %d", g, h);
 	maximise=CL->maximise;
 /********************************/	
 /*CLEAN UP AFTER USE*/
@@ -430,14 +431,16 @@ int gotoh_pair_wise_sw (Alignment *A, int*ns, int **l_s,Constraint_list *CL)
 	
 	aln=A->seq_al;
 	
-	
 	for ( c=0; c<2; c++)
 	    for ( a=0; a<ns[c]; a++)
 	        {
-		e=(c==0)?last_i:last_j;
-		for ( d=0; d<e; d++)
+		  A->order[l_s[c][a]][2]=(c==0)?last_i:last_j;
+		  A->order[l_s[c][a]][3]=(c==1)?best_i:best_j;
+		  
+		  e=(c==0)?last_i:last_j;
+		  for ( d=0; d<e; d++)
 		    {
-		    A->order[l_s[c][a]][1]+=1-is_gap(aln[l_s[c][a]][d]);
+		      A->order[l_s[c][a]][1]+=1-is_gap(aln[l_s[c][a]][d]);
 		    }
 		}
 	
@@ -487,77 +490,6 @@ int gotoh_pair_wise_sw (Alignment *A, int*ns, int **l_s,Constraint_list *CL)
 /*	                                                                       */
 /*******************************************************************************/
 
-double compute_penalty   (Constraint_list *CL, char *mode, int len)
-     {
-
-
-     double p;
-     int **bin;
-     int coor;
-     int n;
-
-     bin=bin_list (CL, WE,0);
-     n=read_size_int ( bin,sizeof (int*));
-     p=return_max_coor_int(bin, n, 3, &coor);     
-     
-     
-     p=bin[coor][3];
-     free_int (bin, -1);
-     
-     return p;
-     }
-
-
-
-double compute_scale ( Constraint_list *CL,char *mode, int len)
-     {
-     double p;
-     int n;
-     int **bin;
-     int coor;
-
-    
-     bin=bin_list ( CL,WE,0);
-     n=read_size_int ( bin,sizeof (int*)); 
-     p=return_max_coor_int(bin, n, 3, &coor);     
-     p=bin[1][3];
-  
-     free_int (bin, -1);
-     return p;
-          
-     }
-	 
-
-
-
-
-int evaluate_penalty (Alignment *A, Constraint_list *CL, int *scale,char *scale_mode, int *penalty, char *penalty_mode, int len_seq)
-    {
-    Constraint_list *NCL;
-
-    if ( scale[0]!=0 && penalty[0]!=0)return 1;
-    else
-        {
-	if (CL->M)
-	    {
-	    scale[0]=0;
-	    penalty[0]=-0.5*SCORE_K;
-	    }
-	else
-	    {
-	    
-	    NCL=duplicate_constraint_list ( CL);
-	    NCL=mask_list_with_aln_pair ( A, 0, (A==NULL)?0:A->len_aln,NCL, UNDEFINED);
-	    scale[0]=compute_scale ( NCL, scale_mode, len_seq)*-SCORE_K;	    
-	    penalty[0]=compute_penalty ( NCL, penalty_mode, len_seq)*-SCORE_K;	
-	    free_constraint_list (NCL);   
-	    }
-	return 1;
-	}
-    }
-
-
-
 Alignment * add_seq2aln   (Constraint_list *CL, Alignment *IN,Sequence  *S)
            {	
 	   int *n_groups;
@@ -626,7 +558,7 @@ Alignment * add_seq2aln   (Constraint_list *CL, Alignment *IN,Sequence  *S)
 /* Centre National de la Recherche Scientifique (CNRS) */
 /*and */
 /*Cedric Notredame */
-/*Tue May 10 12:08:44     2005. */
+/*Fri Oct 26 17:03:04     2007. */
 /*All rights reserved.*/
 /*This file is part of T-COFFEE.*/
 /**/
