@@ -10,10 +10,9 @@
 /* extern char name0[], name1[]; */
 /* extern int match, mismh; */
 
-void* sim_vcalloc( size_t nobj, size_t size);
-void sim_free_all (); 
-int sim_reset_static_variable ();
-Constraint_list *CL;
+
+
+static Constraint_list *CL;
 static int * ns;
 static int **l_s;
 static Alignment *Aln;
@@ -21,10 +20,19 @@ static int **pos;
 static int *seqc0, *seqc1;
 static int min0,min1,max0,max1,mins;
 
+static void* sim_vcalloc( size_t nobj, size_t size);
+static void sim_free_all (); 
+static int sim_reset_static_variable ();
+static int big_pass(int *A,int *B,int M,int N,int K, int nseq) ;
+static int locate(int *A,int *B,int nseq); 
+static int small_pass(int *A,int *B,int count,int nseq);
+static int no_cross ();
+static int diff_sim( int *A,int *B,int M,int N,int tb,int te); 
+int calcons(int *aa0,int n0,int *aa1,int n1,int *res,int *nc,int *nident, Alignment *A, int *ns, int **l_s, Constraint_list *CL);
 
-
-int gscore;
-#define TC_SCORE(x,y) ((CL->get_dp_cost)(Aln, pos, ns[0], l_s[0], x, pos, ns[1], l_s[1], y, CL))
+#define TC_SCORE_2(x,y) (SCORE_K*CL->M[Aln->seq_al[l_s[0][0]][x]-'A'][Aln->seq_al[l_s[1][0]][y]-'A']-SCORE_K*CL->nomatch) 
+#define TC_SCORE_N(x,y) ((CL->get_dp_cost)(Aln, pos, ns[0], l_s[0], x, pos, ns[1], l_s[1], y, CL))
+#define TC_SCORE(x,y)  TC_SCORE_N(x,y)
 
 #define SIM_GAP -1
 #define min(x,y) ((x)<=(y) ? (x) : (y))
@@ -34,6 +42,7 @@ static int q, r;			/* gap penalties */
 static int qr;				/* qr = q + r */
 
 
+
 typedef struct ONE { int COL ;  struct ONE  *NEXT ;} pair, *pairptr;
 pairptr *row, z, z1; 			/* for saving used aligned pairs */
 
@@ -96,9 +105,7 @@ static int flag;			/* indicate if recomputation necessary*/
 
 /* The following definitions are for function diff() */
 
-static int  diff_sim(), display(),calcons();
 static int  zero = 0;				/* int type zero        */
-
 #define gap(k)  ((k) <= 0 ? 0 : q+r*(k))	/* k-symbol indel score */
 
 static int *sapp;				/* Current script append ptr */
@@ -109,28 +116,28 @@ static int no_mat; 				/* number of matches */
 static int no_mis; 				/* number of mismatches */ 
 static int al_len; 				/* length of alignment */
 						/* Append "Delete k" op */
-#define DEL(k)				\
-{ I += k;				\
-  al_len += k;				\
-  if (last < 0)				\
-    last = sapp[-1] -= (k);		\
-  else					\
-    last = *sapp++ = -(k);		\
+#define DEL(k) \
+{ I += k;\
+  al_len += k;\
+  if (last < 0)\
+    last = sapp[-1] -= (k);\
+  else\
+    last = *sapp++ = -(k);\
 }
 						/* Append "Insert k" op */
-#define INS(k)				\
-{ J += k;				\
-  al_len += k;				\
-  if (last < 0)				\
+#define INS(k) \
+{ J += k;\
+  al_len += k;\
+  if (last < 0)\
     { sapp[-1] = (k); *sapp++ = last; }	\
-  else					\
-    last = *sapp++ = (k);		\
+  else\
+    last = *sapp++ = (k);\
 }
 
 						/* Append "Replace" op */
-#define REP 				\
-{ last = *sapp++ = 0; 			\
-  al_len += 1;				\
+#define REP \
+{ last = *sapp++ = 0;\
+  al_len += 1;\
 }
 
 
@@ -173,7 +180,7 @@ int sim_pair_wise_lalign (Alignment *in_A, int *in_ns, int **in_l_s,Constraint_l
   int R, Q;
   Alignment *DA;
   
-  
+
   DA=in_A;
 
   Aln=copy_aln (in_A, NULL);
@@ -365,14 +372,6 @@ int sim_pair_wise_lalign (Alignment *in_A, int *in_ns, int **in_l_s,Constraint_l
   padd_aln (in_A);
   
   sim_free_all();
-  
-  /*reset every static vraible before exit
-  CC=DD=RR=SS=EE=FF=HH=WW=II=JJ=XX=YY=sapp=NULL;
-  min0=min1=max0=max1=mins=q=r=qr=tt=numnode=m1=n1=nn=rl=cl=lmin=flag=zero=last=I=J=no_mat=no_mis=al_len=0;
-  most=low=NULL;Very important: cause a bug if not reset
-  LIST=NULL;    Very important: cause a bug if not reset
-  */
-  
   free_int (pos, -1);
   free_aln (Aln);
 
@@ -392,7 +391,7 @@ int sim_reset_static_variable ()
 /* A big pass to compute K best classes */
 
 
-big_pass(A,B,M,N,K,nseq) int A[],B[]; int M,N,K,nseq;
+int big_pass(int *A,int *B,int M,int N,int K, int nseq) 
 { register  int  i, j;			/* row and column indices */
   register  int  c;			/* best score at current point */
   register  int  f;			/* best score ending with insertion */
@@ -449,6 +448,7 @@ big_pass(A,B,M,N,K,nseq) int A[],B[]; int M,N,K,nseq;
 		  c = 0;
 		 
 		  DIAG(i, j, c, p+TC_SCORE(A[i-1],B[j-1]))		/* diagonal */
+		    
 		  if ( c <= 0 )
 		    { c = 0; ci = i; cj = j; }
 		  else
@@ -473,14 +473,14 @@ return 1;
 
 /* Determine the left and top boundaries of the recomputed area */
 
-locate(A,B,nseq) int A[],B[]; int nseq;
+int locate(int *A,int *B,int nseq) 
 { register  int  i, j;			/* row and column indices */
   register  int  c;			/* best score at current point */
   register  int  f;			/* best score ending with insertion */
   register  int  d;			/* best score ending with deletion */
   register  int  p;			/* best score at (i-1, j-1) */
   register  int  ci, cj;		/* end-point associated with c */ 
-  register  int  di, dj;		/* end-point associated with d */
+  register  int  di=0, dj=0;		/* end-point associated with d */
   register  int  fi, fj;		/* end-point associated with f */
   register  int  pi, pj;		/* end-point associated with p */
   int  cflag, rflag;			/* for recomputation */
@@ -535,6 +535,7 @@ locate(A,B,nseq) int A[],B[]; int nseq;
 		  ORDER(d, di, dj, c, ci, cj)
 		  c = 0;
 		  DIAG(i, j, c, p+TC_SCORE(A[i-1],B[j-1]))		/* diagonal */
+		    
 		  if ( c <= 0 )
 		    { c = 0; ci = i; cj = j; }
 		  else
@@ -587,6 +588,7 @@ locate(A,B,nseq) int A[],B[]; int nseq;
 		  ORDER(d, di, dj, c, ci, cj)
 		  c = 0;
 		  DIAG(m1, j, c, TC_SCORE(A[m1-1],B[j-1]))		/* diagonal */
+		    
 		  if ( c <= 0 )
 		    { c = 0; ci = m1; cj = j; }
 		  else
@@ -604,8 +606,8 @@ locate(A,B,nseq) int A[],B[]; int nseq;
 		  FF[j] = dj;
 		  if ( c > lmin )
 		     flag = 1;
-		  if ( ! rflag && ( ci > rl && cj > cl || di > rl && dj > cl
-	 		                            || fi > rl && fj > cl ) )
+		  if ( ! rflag && ( (ci > rl && cj > cl) || (di > rl && dj > cl)
+				    || (fi > rl && fj > cl) ) )
 		      rflag = 1;
 	        }
 	      HH[m1] = CC[n1];
@@ -614,8 +616,8 @@ locate(A,B,nseq) int A[],B[]; int nseq;
 	      WW[m1] = DD[n1];
 	      XX[m1] = SS[n1];
 	      YY[m1] = FF[n1];
-	      if ( ! cflag && ( ci > rl && cj > cl || di > rl && dj > cl
-			     || fi > rl && fj > cl ) )
+	      if ( ! cflag && ( (ci > rl && cj > cl) || (di > rl && dj > cl)
+				|| (fi > rl && fj > cl )) )
 	         cflag = 1;
 	    }
 
@@ -653,6 +655,8 @@ locate(A,B,nseq) int A[],B[]; int nseq;
 		  ORDER(d, di, dj, c, ci, cj)
 		  c = 0;
 	          DIAG(i, n1, c, p+TC_SCORE(A[i-1], B[n1-1]))
+		    
+		  
 		  if ( c <= 0 )
 		    { c = 0; ci = i; cj = n1; }
 		  else
@@ -670,8 +674,8 @@ locate(A,B,nseq) int A[],B[]; int nseq;
 		  YY[i] = dj;
 		  if ( c > lmin )
 		     flag = 1;
-	          if ( ! cflag && ( ci > rl && cj > cl || di > rl && dj > cl
-		               || fi > rl && fj > cl ) )
+	          if ( ! cflag && ( (ci > rl && cj > cl) || (di > rl && dj > cl)
+				    || (fi > rl && fj > cl )) )
 		     cflag = 1;
 	        }
 	      CC[n1] = HH[m1];
@@ -680,12 +684,12 @@ locate(A,B,nseq) int A[],B[]; int nseq;
 	      DD[n1] = WW[m1];
 	      SS[n1] = XX[m1];
 	      FF[n1] = YY[m1];
-	      if ( ! rflag && ( ci > rl && cj > cl || di > rl && dj > cl
-		                                 || fi > rl && fj > cl ) )
+	      if ( ! rflag && ( (ci > rl && cj > cl) || (di > rl && dj > cl)
+				|| (fi > rl && fj > cl) ) )
 	         rflag = 1;
 	    }
 	}
-      if ( m1 == 1 && n1 == 1 || no_cross() )
+      if ( (m1 == 1 && n1 == 1) || no_cross() )
 	 break;
    }
   m1--;
@@ -694,7 +698,7 @@ return 1;
 }
 
 /* recompute the area on forward pass */
-small_pass(A,B,count,nseq) int A[], B[]; int count, nseq;
+int small_pass(int *A,int *B,int count,int nseq)
 { register  int  i, j;			/* row and column indices */
   register  int  c;			/* best score at current point */
   register  int  f;			/* best score ending with insertion */
@@ -746,6 +750,7 @@ small_pass(A,B,count,nseq) int A[], B[]; int count, nseq;
 		  ORDER(d, di, dj, c, ci, cj)
 		  c = 0;
 		  DIAG(i, j, c, p+TC_SCORE(A[i-1], B[j-1]))		/* diagonal */
+		    
 		  if ( c <= 0 )
 		    { c = 0; ci = i; cj = j; }
 		  else
@@ -842,7 +847,7 @@ vertexptr findmax()
 
 /* return 1 if no node in LIST share vertices with the area */
 
-no_cross()
+int no_cross()
 { vertexptr  cur;
   register int i;
 
@@ -866,137 +871,142 @@ no_cross()
    A[1..M] and B[1..N] that begins(ends) with a delete if tb(te) is zero
    and appends such a conversion to the current script.                   */
 
-int diff_sim(A,B,M,N,tb,te) int *A, *B; int M, N; int tb, te;
+int diff_sim( int *A,int *B,int M,int N,int tb,int te) 
 
 { int   midi, midj, type;	/* Midpoint, type, and cost */
   int midc;
 
-{ register int   i, j;
-  register int c, e, d, s;
-           int t;
-
-
-/* Boundary cases: M <= 1 or N == 0 */
-
-  if (N <= 0)
-    { if (M > 0) DEL(M)
-      return - gap(M);
-    }
-  if (M <= 1)
-    { if (M <= 0)
-        { INS(N);
-          return - gap(N);
-        }
-      if (tb > te) tb = te;
-      midc = - (tb + r + gap(N) );
-      midj = 0;
-
-      for (j = 1; j <= N; j++)
-        {  for ( tt = 1, z = row[I+1]; z != PAIRNULL; z = z->NEXT )	
-              if ( z->COL == j+J )			
-	         { tt = 0; break; }		
-           if ( tt )			
-            { c = TC_SCORE (A[0],B[j-1]) - ( gap(j-1) + gap(N-j) );
-              if (c > midc)
-               { midc = c;
-                 midj = j;
-               }
-	    }
-	}
-      if (midj == 0)
-        { INS(N) DEL(1) }
-      else
-        { if (midj > 1) INS(midj-1)
-          REP
-	  if ( A[1] == B[midj] )
-	     no_mat += 1;
-	  else
-	     no_mis += 1;
-	  /* mark (A[I],B[J]) as used: put J into list row[I] */	
-          I++; J++;
-	  
-	  
-	  z = ( pairptr )sim_vcalloc(1,sizeof(pair));
-          z->COL = J;			
-          z->NEXT = row[I];				
-	  row[I] = z;
-          if (midj < N) INS(N-midj)
-        }
-      return midc;
-    }
-
-/* Divide: Find optimum midpoint (midi,midj) of cost midc */
-
-  midi = M/2;			/* Forward phase:                          */
-  CC[0] = 0;			/*   Compute C(M/2,k) & D(M/2,k) for all k */
-  t = -q;
-  for (j = 1; j <= N; j++)
-    { CC[j] = t = t-r;
-      DD[j] = t-q;
-    }
-  t = -tb;
-  for (i = 1; i <= midi; i++)
-    { s = CC[0];
-      CC[0] = c = t = t-r;
-      e = t-q;
-
-      for (j = 1; j <= N; j++)
-        { if ((c = c - qr) > (e = e - r)) e = c;
-          if ((c = CC[j] - qr) > (d = DD[j] - r)) d = c;
-	  DIAG(i+I, j+J, c, s+TC_SCORE(A[i-1], B[j-1]))
-          if (c < d) c = d;
-          if (c < e) c = e;
-          s = CC[j];
-          CC[j] = c;
-          DD[j] = d;
-        }
-    }
-  DD[0] = CC[0];
-
-  RR[N] = 0;			/* Reverse phase:                          */
-  t = -q;			/*   Compute R(M/2,k) & S(M/2,k) for all k */
-  for (j = N-1; j >= 0; j--)
-    { RR[j] = t = t-r;
-      SS[j] = t-q;
-    }
-  t = -te;
-  for (i = M-1; i >= midi; i--)
-    { s = RR[N];
-      RR[N] = c = t = t-r;
-      e = t-q;
-
-      for (j = N-1; j >= 0; j--)
-        { if ((c = c - qr) > (e = e - r)) e = c;
-          if ((c = RR[j] - qr) > (d = SS[j] - r)) d = c;
-	  DIAG(i+1+I, j+1+J, c, s+TC_SCORE(A[i],B[j])) /*not -1 on purpose*/
-          if (c < d) c = d;
-          if (c < e) c = e;
-          s = RR[j];
-          RR[j] = c;
-          SS[j] = d;
-        }
-    }
-  SS[N] = RR[N];
-
-  midc = CC[0]+RR[0];		/* Find optimal midpoint */
-  midj = 0;
-  type = 1;
-  for (j = 0; j <= N; j++)
-    if ((c = CC[j] + RR[j]) >= midc)
-      if (c > midc || CC[j] != DD[j] && RR[j] == SS[j])
-        { midc = c;
-          midj = j;
-        }
-  for (j = N; j >= 0; j--)
-    if ((c = DD[j] + SS[j] + q) > midc)
-      { midc = c;
-        midj = j;
-        type = 2;
+  {
+    register int   i, j;
+    register int c, e, d, s;
+    int t;
+    
+    
+    /* Boundary cases: M <= 1 or N == 0 */
+    
+    if (N <= 0)
+      { if (M > 0) DEL(M)
+	  return - gap(M);
       }
-}
-
-/* Conquer: recursively around midpoint */
-
+    if (M <= 1)
+      { if (M <= 0)
+	  { INS(N);
+	    return - gap(N);
+	  }
+	if (tb > te) tb = te;
+	midc = - (tb + r + gap(N) );
+	midj = 0;
+	
+	for (j = 1; j <= N; j++)
+	  {  for ( tt = 1, z = row[I+1]; z != PAIRNULL; z = z->NEXT )	
+              if ( z->COL == j+J )			
+		{ tt = 0; break; }		
+	    if ( tt )			
+	      { c = TC_SCORE (A[0],B[j-1]) - ( gap(j-1) + gap(N-j) );
+		
+		if (c > midc)
+		  { midc = c;
+		    midj = j;
+		  }
+	      }
+	  }
+	if (midj == 0)
+	  { INS(N) DEL(1) }
+	else
+	  { if (midj > 1) INS(midj-1)
+	      REP
+	      if ( A[1] == B[midj] )
+		no_mat += 1;
+	      else
+		no_mis += 1;
+	    /* mark (A[I],B[J]) as used: put J into list row[I] */	
+	    I++; J++;
+	    
+	    
+	    z = ( pairptr )sim_vcalloc(1,sizeof(pair));
+	    z->COL = J;			
+	    z->NEXT = row[I];				
+	    row[I] = z;
+	    if (midj < N) INS(N-midj)
+	      }
+	return midc;
+      }
+    
+    /* Divide: Find optimum midpoint (midi,midj) of cost midc */
+    
+    midi = M/2;			/* Forward phase:                          */
+    CC[0] = 0;			/*   Compute C(M/2,k) & D(M/2,k) for all k */
+    t = -q;
+    for (j = 1; j <= N; j++)
+      { CC[j] = t = t-r;
+	DD[j] = t-q;
+      }
+    t = -tb;
+    for (i = 1; i <= midi; i++)
+      { s = CC[0];
+	CC[0] = c = t = t-r;
+	e = t-q;
+	
+	for (j = 1; j <= N; j++)
+	  { if ((c = c - qr) > (e = e - r)) e = c;
+	    if ((c = CC[j] - qr) > (d = DD[j] - r)) d = c;
+	    DIAG(i+I, j+J, c, s+TC_SCORE(A[i-1], B[j-1]))
+	      
+	      
+	      if (c < d) c = d;
+	    if (c < e) c = e;
+	    s = CC[j];
+	    CC[j] = c;
+	    DD[j] = d;
+	  }
+      }
+    DD[0] = CC[0];
+    
+    RR[N] = 0;			/* Reverse phase:                          */
+    t = -q;			/*   Compute R(M/2,k) & S(M/2,k) for all k */
+    for (j = N-1; j >= 0; j--)
+      { RR[j] = t = t-r;
+	SS[j] = t-q;
+      }
+    t = -te;
+    for (i = M-1; i >= midi; i--)
+      { s = RR[N];
+	RR[N] = c = t = t-r;
+	e = t-q;
+	
+	for (j = N-1; j >= 0; j--)
+	  { if ((c = c - qr) > (e = e - r)) e = c;
+	    if ((c = RR[j] - qr) > (d = SS[j] - r)) d = c;
+	    DIAG(i+1+I, j+1+J, c, s+TC_SCORE(A[i],B[j])) /*not -1 on purpose*/
+	      
+	      if (c < d) c = d;
+	    if (c < e) c = e;
+	    s = RR[j];
+	    RR[j] = c;
+	    SS[j] = d;
+	  }
+      }
+    SS[N] = RR[N];
+    
+    midc = CC[0]+RR[0];		/* Find optimal midpoint */
+    midj = 0;
+    type = 1;
+    for (j = 0; j <= N; j++)
+      if ((c = CC[j] + RR[j]) >= midc)
+	if (c > midc || (CC[j] != DD[j] && RR[j] == SS[j]))
+	  { midc = c;
+	    midj = j;
+	  }
+    for (j = N; j >= 0; j--)
+      if ((c = DD[j] + SS[j] + q) > midc)
+	{ midc = c;
+	  midj = j;
+	  type = 2;
+	}
+  }
+  
+  /* Conquer: recursively around midpoint */
+  
   if (type == 1)
     { diff_sim(A,B,midi,midj,tb,q);
       diff_sim(A+midi,B+midj,M-midi,N-midj,q,te);
@@ -1012,7 +1022,7 @@ int diff_sim(A,B,M,N,tb,te) int *A, *B; int M, N; int tb, te;
 	
 
 
-int calcons(int *aa0,int n0,int *aa1,int n1,int *res,int *nc,int *nident)
+int calcons(int *aa0,int n0,int *aa1,int n1,int *res,int *nc,int *nident, Alignment *A, int *ns, int **l_s, Constraint_list *CL)
 {
   int i0, i1;
   int op, nid, lenc, nd;
@@ -1125,7 +1135,7 @@ void sim_free_all()
 /* Centre National de la Recherche Scientifique (CNRS) */
 /*and */
 /*Cedric Notredame */
-/*Tue May 10 12:08:44     2005. */
+/*Fri Oct 26 17:03:04     2007. */
 /*All rights reserved.*/
 /*This file is part of T-COFFEE.*/
 /**/
